package com.bocloud.tse.booter.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.tse.intf.EngineService;
import com.bocloud.tse.model.EngineModule;
import com.bocloud.tse.scheduler.core.JobScheduler;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/engines")
@Tag(name = "任务引擎管理")
public class EngineController {

	@Autowired
	private EngineService engineService;
	@Autowired
	private JobScheduler scheduler;

	@Operation(summary = "任务引擎列表")
	@GetMapping
	public GeneralResult<?> services() {
		return engineService.services();
	}

	@Operation(summary = "任务引擎状态")
	@GetMapping("/status")
	public GeneralResult<Boolean> status() {
		return new GeneralResult<>(true, scheduler.isRunning(), "任务引擎状态查询成功");
	}

	@Operation(summary = "任务引擎操作(开启/关闭)")
	@PostMapping("/action")
	public GeneralResult<?> action(@RequestBody EngineModule module) {
		GeneralResult<?> result = engineService.action(module);
		if (result.isSuccess() && engineService.checkInstance(module.getInstance())) {
			String action = module.getAction();
			if ("start".equals(action) && !scheduler.isRunning()) {
				scheduler.switchOn();
			} else if ("stop".equals(action) && scheduler.isRunning()) {
				scheduler.switchOff();
			}
		}
		return result;
	}

	@PutMapping(value = "/exclude")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务引擎排除")
	public GeneralResult<Void> taskExclude(@RequestParam(value = "instance") String instance,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return engineService.exclude(instance, context.getTarget());
	}

	@PutMapping(value = "/recover")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务引擎恢复")
	public GeneralResult<Void> taskRecover(@RequestParam(value = "instance") String instance,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return engineService.recover(instance, context.getTarget());
	}

}
