package com.bocloud.tse.booter.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.intf.TaskInstanceService;
import com.bocloud.tse.model.Job;
import com.bocloud.tse.scheduler.core.JobChannel;
import com.bocloud.tse.scheduler.core.JobScheduler;
import com.bocloud.tse.scheduler.core.TaskCache;
import com.bocloud.tse.scheduler.executor.NodeExecutor;
import com.bocloud.tse.scheduler.executor.TaskExecutor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

/**
 * 状态控制器
 */
@RestController
@RequestMapping("/v1")
@Tag(name = "任务状态管理")
public class SchedulerController {

	@Autowired
	private JobScheduler scheduler;
	@Autowired
	private TaskInstanceService instanceService;
	@Autowired
	private JobChannel channel;

	private TaskCache cache = TaskCache.instance();

	@GetMapping("/status/task/total")
	@Operation(tags = { "CMC" }, summary = "任务整体信息")
	public GeneralResult<Map<String, Object>> totalStatus() {
		Map<String, Object> map = new HashMap<>();
		map.put("RunningTask", cache.getTaskExecutorPool().size());
		map.put("RunningNode", cache.getNodeExecutorPool().size());
		return new GeneralResult<>(true, map, "success");
	}

	@GetMapping("/status/task/running")
	@Operation(tags = { "CMC" }, summary = "执行中任务信息")
	public GeneralResult<Map<String, TaskExecutor>> runningTask() {
		return new GeneralResult<>(true, cache.getTaskExecutorPool(), "success");
	}

	@GetMapping("/status/node/running")
	@Operation(tags = { "CMC" }, summary = "执行中任务节点信息")
	public GeneralResult<Map<String, NodeExecutor>> runningNode() {
		return new GeneralResult<>(true, cache.getNodeExecutorPool(), "success");
	}

	@GetMapping("/status/task/waiting")
	@Operation(tags = { "CMC" }, summary = "等待中任务信息")
	public GeneralResult<List<Job>> waitingTask() {
		return new GeneralResult<>(true, instanceService.getAllJobs(), "success");
	}

	@PostMapping("/task/scheduler")
	@Operation(tags = { "CMC" }, summary = "任务调度处理")
	public GeneralResult<Queue<Job>> scheduler(@RequestBody JSONObject object) {
		String type = object.getString("type");
		if ("insert".equalsIgnoreCase(type)) {
			Job job = JSONObject.parseObject(object.getString("data"), Job.class);
			boolean result = channel.insertTask(job);
			if (!result) {
				return new GeneralResult<>(false, "任务启动失败");
			}
		} else if ("remove".equalsIgnoreCase(type)) {
			Result result = scheduler.removeCronTask(object.getString("data"));
			if (result.isFailed()) {
				return new GeneralResult<>(false, "任务移除失败");
			}
		} else {
			return new GeneralResult<>(false, "暂不支持该类操作");
		}
		return new GeneralResult<>(true, "success");
	}
}
