package com.bocloud.tse.core.domain.args.software;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.annotation.JSONField;
import com.bocloud.tse.core.domain.group.Group;
import com.bocloud.tse.core.domain.group.GroupHost;

import lombok.Data;

@Data
public class MemcacheArg {

    private String version;
    @JSONField(name = "cluster_install")
    private boolean clusterInstall;
    @JSONField(name = "software_config")
    private Map<String, Object> softwareConfig;
    @JSONField(name = "magent_config")
    private Map<String, Object> magentConfig;
    @JSONField(name = "install_telegraf")
    private boolean installTelegraf;

    public MemcacheArg(String version, boolean clusterInstall, Map<String, Object> softwareConfig,
        Map<String, Object> magentConfig, List<Group> groups, boolean installTelegraf) {
        super();
        this.version = version;
        this.clusterInstall = clusterInstall;
        this.softwareConfig = softwareConfig;
        if (clusterInstall) {
            if (null == magentConfig) {
                magentConfig = new HashMap<>();
            }
            if (!magentConfig.containsKey("magent_port")) {
                magentConfig.put("magent_port", 12000);
            }
            List<String> masters = new ArrayList<>();
            List<String> slavers = new ArrayList<>();
            for (Group group : groups) {
                if ("memcached_masters".equals(group.getName())) {
                    List<GroupHost> hosts = group.getHosts();
                    for (GroupHost host : hosts) {
                        masters.add(host.getHost() + ":" + softwareConfig.get("memcached_port"));
                    }
                } else if ("memcached_backups".equals(group.getName())) {
                    List<GroupHost> hosts = group.getHosts();
                    for (GroupHost host : hosts) {
                        slavers.add(host.getHost() + ":" + softwareConfig.get("memcached_port"));
                    }
                }
            }
            magentConfig.put("memcached_servers", masters);
            magentConfig.put("memcached_backups", slavers);
            this.magentConfig = magentConfig;
        }
        this.installTelegraf = installTelegraf;
    }

    public MemcacheArg() {}

}