package com.bocloud.tse.core.domain.args.software;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;

import lombok.Data;

@Data
public class PostgresqlArg {

    private String version;
    @JSONField(name = "cluster_install")
    private boolean clusterInstall;
    @JSONField(name = "software_config")
    private Map<String, Object> softwareConfig;
    @JSONField(name = "postgresql_databases")
    private List<Map<String, Object>> postgresqlDatabases;
    @JSONField(name = "postgresql_users")
    private List<Map<String, Object>> postgresqlUsers;
    @JSONField(name = "postgresql_privileges")
    private List<Map<String, Object>> postgresqlPrivileges;
    @JSONField(name = "postgresql_schemas")
    private List<Map<String, Object>> postgresqlSchemas;
    @JSONField(name = "postgresql_extensions")
    private List<Map<String, Object>> postgresqlExtensions;
    @JSONField(name = "mnt_disks")
    private List<Map<String, Object>> mntDisks;
    @JSONField(name = "etcd_config")
    private Map<String, Object> etcdConfig;
    @JSONField(name = "cluster_config")
    private Map<String, Object> clusterConfig;
    @JSONField(name = "install_telegraf")
    private boolean installTelegraf;
    @JSONField(name = "postgresql_bocloud_user")
    private String postgresqlBocloudUser;
    @JSONField(name = "postgresql_bocloud_password")
    private String postgresqlBocloudPassword;

    public PostgresqlArg(String version, boolean clusterInstall, Map<String, Object> softwareConfig,
        List<Map<String, Object>> postgresqlDatabases, List<Map<String, Object>> postgresqlUsers,
        List<Map<String, Object>> postgresqlPrivileges, List<Map<String, Object>> postgresqlSchemas,
        List<Map<String, Object>> postgresqlExtensions, List<Map<String, Object>> mntDisks,
        Map<String, Object> etcdConfig, Map<String, Object> clusterConfig, boolean installTelegraf,
        String postgresqlBocloudUser, String postgresqlBocloudPassword) {
        super();
        this.version = version;
        this.clusterInstall = clusterInstall;
        this.softwareConfig = softwareConfig;
        this.postgresqlUsers = postgresqlUsers;
        this.postgresqlPrivileges = postgresqlPrivileges;
        this.postgresqlSchemas = postgresqlSchemas;
        this.postgresqlExtensions = postgresqlExtensions;
        if (null != postgresqlDatabases && !postgresqlDatabases.isEmpty()) {
            this.postgresqlDatabases = postgresqlDatabases;
            for (Map<String, Object> databases : this.postgresqlDatabases) {
                Iterator<Entry<String, Object>> iterator = databases.entrySet().iterator();
                while (iterator.hasNext()) {
                    Entry<String, Object> entry = iterator.next();
                    if (null == databases.get(entry.getKey()) || "".equals(databases.get(entry.getKey()))) {
                        iterator.remove();
                    }
                }
            }
        }
        this.mntDisks = mntDisks;
        if (clusterInstall) {
            if (null == etcdConfig || etcdConfig.isEmpty()) {
                etcdConfig = new JSONObject();
                etcdConfig.put("version", "3.1.20");
                etcdConfig.put("cluster_install", true);
                etcdConfig.put("etcd_secure", false);
                etcdConfig.put("etcd_initial_cluster_token", "etcd-cluster");
                JSONObject etcdSoftConfig = new JSONObject();
                etcdSoftConfig.put("etcd_data_dir", "/var/cache/etcd/state");
                etcdSoftConfig.put("etcd_client_port", 2379);
                etcdSoftConfig.put("etcd_peer_port", 2380);
                etcdConfig.put("software_config", etcdSoftConfig);
            }
            if (null == clusterConfig) {
                clusterConfig = new JSONObject();
            }
            clusterConfig.put("with_haproxy_load_balancing", true);
            clusterConfig.put("patroni_etcd_port", 2379);
            clusterConfig.put("patroni_cluster_name", "postgre_cluster");
            this.etcdConfig = etcdConfig;
            this.clusterConfig = clusterConfig;
        }
        this.installTelegraf = installTelegraf;
        this.postgresqlBocloudUser = postgresqlBocloudUser;
        this.postgresqlBocloudPassword = postgresqlBocloudPassword;
    }

    public PostgresqlArg() {}

    public void setPostgresqlDatabases(List<Map<String, Object>> postgresqlDatabases) {
        if (null == postgresqlDatabases || postgresqlDatabases.isEmpty()) {
            return;
        }
        this.postgresqlDatabases = postgresqlDatabases;
    }

    public void setPostgresqlUsers(List<Map<String, Object>> postgresqlUsers) {
        if (null == postgresqlUsers || postgresqlUsers.isEmpty()) {
            return;
        }
        this.postgresqlUsers = postgresqlUsers;
    }

    public void setPostgresqlPrivileges(List<Map<String, Object>> postgresqlPrivileges) {
        if (null == postgresqlPrivileges || postgresqlPrivileges.isEmpty()) {
            return;
        }
        this.postgresqlPrivileges = postgresqlPrivileges;
    }

    public void setPostgresqlSchemas(List<Map<String, Object>> postgresqlSchemas) {
        if (null == postgresqlSchemas || postgresqlSchemas.isEmpty()) {
            return;
        }
        this.postgresqlSchemas = postgresqlSchemas;
    }

    public void setPostgresqlExtensions(List<Map<String, Object>> postgresqlExtensions) {
        if (null == postgresqlExtensions || postgresqlExtensions.isEmpty()) {
            return;
        }
        this.postgresqlExtensions = postgresqlExtensions;
    }

    public List<Map<String, Object>> getMntDisks() {
        return mntDisks;
    }

    public void setMntDisks(List<Map<String, Object>> mntDisks) {
        if (null == mntDisks || mntDisks.isEmpty()) {
            return;
        }
        this.mntDisks = mntDisks;
    }

    public void setEtcdConfig(Map<String, Object> etcdConfig) {
        if (null == etcdConfig || etcdConfig.isEmpty()) {
            return;
        }
        this.etcdConfig = etcdConfig;
    }

    public void setClusterConfig(Map<String, Object> clusterConfig) {
        if (null == clusterConfig || clusterConfig.isEmpty()) {
            return;
        }
        this.clusterConfig = clusterConfig;
    }

}