package com.bocloud.tse.core.domain.group;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.bocloud.tse.core.domain.front.software.ElasticsearchFront;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

public class ElasticsearchGroup {

    private ElasticsearchFront front;
    private List<WorkerHost> targets;
    private Map<String, List<WorkerHost>> targetsMap;
    private List<Group> groups;
    private int index = 1;

    public ElasticsearchGroup(ElasticsearchFront front, List<WorkerHost> targets,
        Map<String, List<WorkerHost>> targetsMap) {
        this.front = front;
        this.targets = targets;
        this.targetsMap = targetsMap;
        this.groups = new ArrayList<>();
        this.build();
    }

    private void build() {
        List<WorkerHost> list = targets;
        // 对数据进行处理
        if (ListTool.isEmpty(list) && null != targetsMap) {
            list = new ArrayList<>();
            Iterator<List<WorkerHost>> iterator = targetsMap.values().iterator();
            while (iterator.hasNext()) {
                list.addAll(iterator.next());
            }
        }
        this.groups.add(buildGroup(list));
    }

    private Group buildGroup(List<WorkerHost> list) {
        Group group = new Group();
        group.setName("es_hosts");
        if (null == list) {
            list = new ArrayList<>();
        }
        List<GroupHost> hosts = new ArrayList<>();
        if (1 == list.size() || 3 == list.size()) {
            // 一个节点， 这个节点为master和data节点
            // 三个节点，三个节点全是master和data节点。
            hosts = buildAllMasterGroup(list);
        } else if (2 == list.size()) {
            // 二个节点，这二个节点都是data节点，仅仅第一个节点是master节点
            hosts = buildFirstMasterGroup(list);
        }
        // 暂时不考虑大于3个节点的情况
        group.setHosts(hosts);
        return group;
    }

    private List<GroupHost> buildAllMasterGroup(List<WorkerHost> workers) {
        List<String> instances = new ArrayList<>();
        List<String> masters = new ArrayList<>();
        List<GroupHost> list = new ArrayList<>();
        workers.stream().forEach(worker -> {
            GroupHost host = new GroupHost();
            host.setHost(worker.getHost());
            // 节点名称规则固定
            String nodeName = "es_node_" + (index++);
            Map<String, Object> vars = MapTools.simpleMap("node_name", nodeName);
            vars.put("node_master", true);
            vars.put("node_data", true);
            host.setVars(vars);
            list.add(host);
            // 拼接software config中参数
            instances.add(worker.getHost() + ":" + worker.getPort());
            masters.add(nodeName);
        });
        // 处理soft config中跟节点相关的配置
        Map<String, Object> config = front.getSoftwareConfig();
        config.put("discovery_seed_hosts", instances);
        config.put("cluster_initial_master_node", masters);
        front.setSoftwareConfig(config);
        return list;
    }

    private List<GroupHost> buildFirstMasterGroup(List<WorkerHost> workers) {
        List<String> instances = new ArrayList<>();
        List<String> masters = new ArrayList<>();
        List<GroupHost> list = new ArrayList<>();
        workers.stream().forEach(worker -> {
            GroupHost host = new GroupHost();
            host.setHost(worker.getHost());
            // 节点名称规则固定
            String nodeName = "es_node_" + index;
            Map<String, Object> vars = MapTools.simpleMap("node_name", nodeName);
            vars.put("node_data", true);
            if (index++ > 1) {
                // 只有第一个节点为master
                vars.put("node_master", false);
            } else {
                vars.put("node_master", true);
                masters.add(nodeName);
            }
            host.setVars(vars);
            list.add(host);
            // 拼接software config中参数
            instances.add(worker.getHost() + ":" + worker.getPort());
        });
        // 处理soft config中跟节点相关的配置
        Map<String, Object> config = front.getSoftwareConfig();
        config.put("discovery_seed_hosts", instances);
        config.put("cluster_initial_master_node", masters);
        front.setSoftwareConfig(config);
        return list;
    }

    public List<Group> getGroups() {
        return groups;
    }
}