package com.bocloud.tse.core.domain.group;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.core.domain.front.software.KafkaFront;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

public class KafkaGroup {

    private KafkaFront front;
    private List<Group> groups;
    private boolean networks;

    public KafkaGroup(KafkaFront front) {
        this.front = front;
        this.groups = new ArrayList<>();
        this.build();
    }

    private void build() {
        List<Group> list = front.getGroups();
        if (ListTool.isEmpty(list)) {
            return;
        }
        for (Group group : list) {
            switch (group.getName()) {
                case "kafka":
                    group = buildKafkaGroup(group);
                    break;
                case "zookeeper":
                    group = buildZkGroup(group);
                    break;
                default:
                    break;
            }
            this.groups.add(group);
        }
    }

    private Group buildKafkaGroup(Group group) {
        List<GroupHost> hosts = group.getHosts();
        for (GroupHost host : hosts) {
            JSONObject object = new JSONObject();
            if (StringUtils.isEmpty(host.getReal()) || host.getHost().equals(host.getReal())) {
                object.put("listeners", "PLAINTEXT://0.0.0.0:9092");
                object.put("advertised_listeners", "PLAINTEXT://" + host.getHost() + ":9092");
            } else {
                object.put("listeners", "INSIDE://0.0.0.0:9093,OUTSIDE://0.0.0.0:9092");
                object.put("advertised_listeners",
                    "OUTSIDE://" + host.getReal() + ":9092,INSIDE://" + host.getHost() + ":9093");
                networks = true;
            }

            Map<String, Object> vars = MapTools.simpleMap("node_server_cfg", object);
            host.setVars(vars);
        }
        return new Group(group.getName(), hosts, group.getVars());
    }

    private Group buildZkGroup(Group group) {
        List<GroupHost> hosts = group.getHosts();
        for (GroupHost host : hosts) {
            if (StringUtils.isEmpty(host.getReal())) {
                Map<String, Object> vars = MapTools.simpleMap("proxy_host", host.getHost());
                host.setVars(vars);
            } else {
                Map<String, Object> vars = MapTools.simpleMap("proxy_host", host.getReal());
                host.setVars(vars);
            }
        }
        return new Group(group.getName(), hosts, group.getVars());
    }

    public List<Group> getGroups() {
        return groups;
    }

    public boolean isNetworks() {
        return networks;
    }

}