package com.bocloud.tse.core.domain.group;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.tse.core.domain.front.software.MongodbFront;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

public class MongoGroup {

    private MongodbFront front;
    private List<Group> groups;

    public MongoGroup(MongodbFront front) {
        this.front = front;
        this.groups = new ArrayList<>();
        this.build();
    }

    private void build() {
        List<Group> list = front.getGroups();
        if (ListTool.isEmpty(list) || !front.isClusterInstall()) {
            return;
        }
        for (Group group : list) {
            switch (group.getName()) {
                case "mongo_master":
                    group = buildMasterGroup(group);
                    break;
                case "mongo_replicas":
                    group = buildSlaveGroup(group);
                    break;
                case "mongo_arbiter":
                    group = buildArbiterGroup(group);
                    break;
                default:
                    continue;
            }
            this.groups.add(group);
        }
    }

    private Group buildMasterGroup(Group group) {
        Map<String, Object> config = front.getManageConfig();
        List<GroupHost> hosts = group.getHosts();
        for (GroupHost host : hosts) {
            Map<String, Object> vars = MapTools.simpleMap("mongodb_master", true);
            List<Map<String, Object>> params = new ArrayList<>();
            if (StringUtils.isEmpty(host.getReal())) {
                params.add(MapTools.simpleMap("host_name", host.getHost()));
                config.put("mongodb_login_host", host.getHost());
            } else {
                params.add(MapTools.simpleMap("host_name", host.getReal()));
                config.put("mongodb_login_host", host.getReal());
            }
            vars.put("mongodb_replication_params", params);
            host.setVars(vars);
            hosts.set(hosts.indexOf(host), host);
        }
        front.setManageConfig(config);
        return new Group(group.getName(), hosts, group.getVars());
    }

    private Group buildSlaveGroup(Group group) {
        List<GroupHost> hosts = group.getHosts();
        for (GroupHost host : hosts) {
            Map<String, Object> vars = new HashMap<>();
            List<Map<String, Object>> params = new ArrayList<>();
            Map<String, Object> param = MapTools.simpleMap("host_type", "replica");
            if (StringUtils.isEmpty(host.getReal())) {
                param.put("host_name", host.getHost());
            } else {
                param.put("host_name", host.getReal());
            }
            params.add(param);
            vars.put("mongodb_replication_params", params);
            host.setVars(vars);
            hosts.set(hosts.indexOf(host), host);
        }
        return new Group(group.getName(), hosts, group.getVars());
    }

    private Group buildArbiterGroup(Group group) {
        List<GroupHost> hosts = group.getHosts();
        for (GroupHost host : hosts) {
            Map<String, Object> vars = new HashMap<>();
            List<Map<String, Object>> params = new ArrayList<>();
            Map<String, Object> param = MapTools.simpleMap("host_type", "arbiter");
            if (StringUtils.isEmpty(host.getReal())) {
                param.put("host_name", host.getHost());
            } else {
                param.put("host_name", host.getReal());
            }
            params.add(param);
            vars.put("mongodb_replication_params", params);
            host.setVars(vars);
            hosts.set(hosts.indexOf(host), host);
        }
        return new Group(group.getName(), hosts, group.getVars());
    }

    public List<Group> getGroups() {
        return groups;
    }

}