package com.bocloud.tse.core.domain.group;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.tse.core.domain.front.software.MysqlFront;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

public class MysqlGroup {

    private MysqlFront front;
    private List<Group> groups;
    private int index = 1;

    public MysqlGroup(MysqlFront front) {
        this.front = front;
        this.groups = new ArrayList<>();
        this.build();
    }

    private void build() {
        List<Group> list = front.getGroups();
        if (!front.isClusterInstall() || ListTool.isEmpty(list)) {
            return;
        }
        for (Group group : list) {
            switch (group.getName()) {
                case "mysql_master":
                    group = buildMasterGroup(group);
                    break;
                case "mysql_slave":
                    group = buildSlaveGroup(group);
                    break;
                case "mysql":
                    group = buildMysqlGroup(group);
                    break;
                default:
                    continue;
            }
            this.groups.add(group);
        }
        if (front.isClusterInstall()) {
            boolean hasMysql = false;
            List<String> hosts = new ArrayList<>();
            for (Group group : groups) {
                if ("mysql".equalsIgnoreCase(group.getName())) {
                    hasMysql = true;
                }
                List<GroupHost> groupHosts = group.getHosts();
                for (GroupHost host : groupHosts) {
                    if (!hosts.contains(host.getHost())) {
                        hosts.add(host.getHost());
                    }
                }
            }
            if (!hasMysql) {
                List<GroupHost> groupHosts = new ArrayList<>();
                for (String host : hosts) {
                    groupHosts.add(new GroupHost(host, null));
                }
                groups.add(new Group("mysql", groupHosts, null));
            }
        }
        String type = front.getClusterType();
        if ("msb".equalsIgnoreCase(type) || "mgr.single".equalsIgnoreCase(type)) {
            String vip = front.getClusterVip();
            if (StringUtils.isNotEmpty(vip)) {
                front.setKeepalivedVirtualRouterId(Integer.valueOf(vip.substring(vip.lastIndexOf(".") + 1)));
            }
        }
        if (type.startsWith("mgr")) {
            Map<String, Object> config = front.getSoftwareConfig();
            config.put("mysql_group_replication_single_primary_mode",
                type.equalsIgnoreCase("mgr.single") ? "ON" : "OFF");
            config.put("mysql_group_replication_enforce_update_everywhere_checks",
                type.equalsIgnoreCase("mgr.single") ? "OFF" : "ON");
            front.setSoftwareConfig(config);
            front.setClusterType("mgr");
        }
    }

    private Group buildMasterGroup(Group group) {
        if (front.getClusterType().equalsIgnoreCase("mgr.multi")) {
            List<GroupHost> hosts = group.getHosts();
            for (GroupHost host : hosts) {
                Map<String, Object> vars = MapTools.simpleMap("mysql_server_id", index);
                vars.put("mysql_replication_role", index == 1 ? "master" : "second_master");
                vars.put("public_ip", StringUtils.isEmpty(host.getReal()) ? host.getHost() : host.getReal());
                host.setVars(vars);
                index++;
            }
            return group;
        }
        GroupHost host = group.getHosts().get(0);
        String address = StringUtils.isEmpty(host.getReal()) ? host.getHost() : host.getReal();
        Map<String, Object> vars = MapTools.simpleMap("mysql_server_id", index);
        vars.put("mysql_replication_role", "master");
        vars.put("public_ip", address);
        host.setVars(vars);
        Map<String, Object> config = front.getSoftwareConfig();
        config.put("replication_master_host", address);
        front.setSoftwareConfig(config);
        index++;
        return group;
    }

    private Group buildSlaveGroup(Group group) {
        List<GroupHost> hosts = group.getHosts();
        for (GroupHost host : hosts) {
            Map<String, Object> vars = MapTools.simpleMap("mysql_server_id", index);
            vars.put("mysql_replication_role", "slave");
            if (front.getClusterType().equalsIgnoreCase("mha") && hosts.indexOf(host) == 0) {
                vars.put("is_manager", true);
            }
            vars.put("public_ip", StringUtils.isEmpty(host.getReal()) ? host.getHost() : host.getReal());
            host.setVars(vars);
            index++;
        }
        return group;
    }

    private Group buildMysqlGroup(Group group) {
        return group;
    }

    public List<Group> getGroups() {
        return groups;
    }

}