package com.bocloud.tse.core.domain.group;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.tse.core.domain.front.software.PostgresqlFront;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

public class PostgresqlGroup {

    private PostgresqlFront front;
    private List<Group> groups;

    public PostgresqlGroup(PostgresqlFront front) {
        this.front = front;
        this.groups = new ArrayList<>();
        this.build();
    }

    private void build() {
        List<Group> list = front.getGroups();
        if (ListTool.isEmpty(list)) {
            return;
        }
        for (Group group : list) {
            switch (group.getName()) {
                case "postgres_cluster":
                    group = buildGroup(group);
                    List<GroupHost> hosts = group.getHosts();
                    List<GroupHost> etcdHosts = new ArrayList<>();
                    for (GroupHost host : hosts) {
                        GroupHost groupHost = new GroupHost();
                        groupHost.setHost(host.getHost());
                        groupHost.setReal(host.getReal());
                        etcdHosts.add(groupHost);
                    }
                    this.groups.add(new Group("etcd_hosts", etcdHosts, null));
                    break;
                default:
                    break;
            }
            this.groups.add(group);
        }
    }

    private Group buildGroup(Group group) {
        List<GroupHost> hosts = group.getHosts();
        if (!ListTool.isEmpty(hosts)) {
            int index = 1;
            List<String> etcdHosts = new ArrayList<>();
            for (GroupHost host : hosts) {
                Map<String, Object> vars = MapTools.simpleMap("is_master", index == 1);
                vars.put("hostname", "pgnode_" + index);
                host.setVars(vars);
                index++;
                etcdHosts.add(StringUtils.isEmpty(host.getReal()) ? host.getHost() : host.getReal());
            }
            Map<String, Object> clusterConfig = front.getClusterConfig();
            if (null == clusterConfig) {
                clusterConfig = new HashMap<>();
            }
            clusterConfig.put("patroni_etcd_hosts", etcdHosts);
            front.setClusterConfig(clusterConfig);
        }
        return new Group(group.getName(), hosts, group.getVars());
    }

    public List<Group> getGroups() {
        return groups;
    }

}