package com.bocloud.tse.core.domain.group;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.tse.core.domain.front.software.RedisFront;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

public class RedisGroup {

    private List<GroupHost> masters;
    private List<Group> groups;
    private RedisFront front;

    public RedisGroup(RedisFront front) {
        this.front = front;
        this.groups = new ArrayList<>();
        this.build();
    }

    private void build() {
        List<Group> list = front.getGroups();
        if (ListTool.isEmpty(list)) {
            return;
        }
        for (Group group : list) {
            switch (group.getName()) {
                case "redis_master":
                    this.masters = group.getHosts();
                    group = buildMasterGroup(group);
                    break;
                case "redis_slave":
                    group = buildSlaveGroup(group, front.getSoftwareConfig());
                    break;
                case "redis_sentinel":
                    group.setVars(MapTools.simpleMap("redis_sentinel", true));
                    break;
                default:
                    continue;
            }
            if (!ListTool.isEmpty(group.getHosts())) {
                this.groups.add(group);
            }
        }
    }

    private Group buildMasterGroup(Group group) {
        List<GroupHost> hosts = group.getHosts();
        List<String> nodes = new ArrayList<>();
        for (GroupHost host : hosts) {
            String proxyHost = StringUtils.isEmpty(host.getReal()) ? host.getHost() : host.getReal();
            Map<String, Object> vars = MapTools.simpleMap("proxy_host", proxyHost);
            vars.put("is_master", true);
            host.setVars(vars);
            hosts.set(hosts.indexOf(host), host);
            nodes.add(proxyHost);
        }
        front.setRedisMasterNodes(nodes);
        return new Group(group.getName(), hosts, group.getVars());
    }

    private Group buildSlaveGroup(Group group, Map<String, Object> config) {
        List<GroupHost> hosts = group.getHosts();
        int size = this.masters.size();
        String port = String.valueOf(config.get("redis_port"));
        if (StringUtils.isEmpty(port)) {
            port = "6379";
        }
        List<String> nodes = new ArrayList<>();
        for (GroupHost host : hosts) {
            if (StringUtils.isEmpty(host.getReal())) {
                String master = masters.get(hosts.indexOf(host) % size).getHost();
                Map<String, Object> vars = MapTools.simpleMap("redis_slaveof", master + " " + port);
                vars.put("proxy_host", host.getHost());
                vars.put("is_slave", true);
                host.setVars(vars);
                nodes.add(host.getHost());
            } else {
                String master = masters.get(hosts.indexOf(host) % size).getReal();
                Map<String, Object> vars = MapTools.simpleMap("redis_slaveof", master + " " + port);
                vars.put("proxy_host", host.getReal());
                vars.put("is_slave", true);
                host.setVars(vars);
                nodes.add(host.getReal());
            }
            hosts.set(hosts.indexOf(host), host);
        }
        front.setRedisSlaveNodes(nodes);
        return new Group(group.getName(), hosts, group.getVars());
    }

    public List<Group> getGroups() {
        return groups;
    }

}