package com.bocloud.tse.core.domain.group;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bocloud.tse.core.domain.front.software.RocketmqFront;
import com.megatron.common.utils.ListTool;

public class RocketmqGroup {

    private RocketmqFront front;
    private List<Group> groups;

    public RocketmqGroup(RocketmqFront front) {
        this.front = front;
        this.groups = new ArrayList<>();
        this.build();
    }

    private void build() {
        List<Group> list = front.getGroups();
        if (ListTool.isEmpty(list)) {
            return;
        }
        List<GroupHost> hosts = new ArrayList<>();
        for (Group group : list) {
            switch (group.getName()) {
                case "rocketmq_master":
                    hosts.add(buildMasterHost(group));
                    break;
                case "rocketmq_slaver":
                    hosts.addAll(buildSlaverHost(group));
                    break;
                case "rocketmq_namesrv":
                    this.groups.add(buildNSGroup(group));
                    break;
                default:
                    continue;
            }
        }
        this.groups.add(new Group("rocketmq_broker", hosts, null));
    }

    private GroupHost buildMasterHost(Group group) {
        Map<String, Object> vars = new HashMap<>();
        vars.put("broker_id", "0");
        vars.put("broker_role", "ASYNC_MASTER");
        vars.put("broker_name", "broker_" + front.getClusterName());
        return new GroupHost(group.getHosts().get(0).getHost(), vars);
    }

    private List<GroupHost> buildSlaverHost(Group group) {
        List<GroupHost> hosts = group.getHosts();
        List<GroupHost> list = new ArrayList<>();
        int index = 1;
        for (GroupHost host : hosts) {
            Map<String, Object> vars = new HashMap<>();
            vars.put("broker_id", index + "");
            vars.put("broker_role", "SLAVE");
            vars.put("broker_name", "broker_" + front.getClusterName());
            list.add(new GroupHost(host.getHost(), vars));
            index++;
        }
        return list;
    }

    private Group buildNSGroup(Group group) {
        return group;
    }

    public List<Group> getGroups() {
        return groups;
    }

}