package com.bocloud.tse.core.utils;

public class DBTool {

    /**
     * 获取mysql的分页语句 pageNo 从0 开始
     */
    public static String buildMysqlPageSql(String sql, Integer pageNo, Integer pageSize) {
        StringBuilder sqlBuilder = new StringBuilder(sql.length() + 14);
        sqlBuilder.append(sql);
        if (pageNo == 0) {
            sqlBuilder.append(" LIMIT ").append(pageSize);
        } else {
            sqlBuilder.append(" LIMIT ").append(pageNo).append(",").append(pageSize);
        }
        return sqlBuilder.toString();
    }

    /**
     * 获取oracle的分页语句 pageNo 从0 开始
     */
    public static String buildOraclePageSql(String sql, Integer pageNo, Integer pageSize) {
        int startRow = pageNo;
        int endRow = pageNo + pageSize;
        StringBuilder sqlBuilder = new StringBuilder(sql.length() + 120);
        if (startRow > 0) {
            sqlBuilder.append("SELECT * FROM ( ");
        }
        if (endRow > 0) {
            sqlBuilder.append(" SELECT TMP_PAGE.*, ROWNUM ROW_ID FROM ( ");
        }
        sqlBuilder.append(sql);
        if (endRow > 0) {
            sqlBuilder.append(" ) TMP_PAGE WHERE ROWNUM <= ").append(endRow);
        }
        if (startRow > 0) {
            sqlBuilder.append(" ) WHERE ROW_ID > ").append(startRow);
        }
        return sqlBuilder.toString();
    }

    /**
     * 获取sqlserver2012的分页语句 pageNo 由 0 开始
     */
    public static String buildSqlServer2012PageSql(String sql, Integer pageNo, Integer pageSize) {
        StringBuilder sqlBuilder = new StringBuilder(sql.length() + 14);
        sqlBuilder.append(sql).append(" OFFSET ").append(pageNo).append(" ROWS ").append(" FETCH NEXT ")
            .append(pageSize).append(" ROWS ONLY");
        return sqlBuilder.toString();
    }

    public static boolean isSqlServer(String driver) {
        return DBDriver.SQLSERVER_JTS.getDriver().equals(driver) || DBDriver.SQLSERVER_MS.getDriver().equals(driver);
    }

    public static boolean isOracle(String driver) {
        return DBDriver.ORACLE.getDriver().equals(driver);
    }

    public static boolean isMySql(String driver) {
        return DBDriver.MYSQL.getDriver().equals(driver);
    }

}