package com.bocloud.tse.entity;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

/**
 * 任务节点类
 */
@Data
@EqualsAndHashCode(callSuper = true)
@AllArgsConstructor
@NoArgsConstructor
@Table("task_node")
public class TaskNode extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("uuid")
    private String uuid; // 任务ID
    @Column("code")
    private String code; // 节点编码
    @Column("category")
    private String category; // 节点类型
    @Column("catalog")
    private String catalog; // 节点分类
    @Column("input")
    private String input; // 参数
    @Column("output")
    private String output; // 输出
    @Column("content")
    private String content; // 默认参数
    @Column("rollbacks")
    private String rollbacks;// 回滚
    @Column("config")
    private String config; // 配置信息
    @Column("control")
    private String control;// 控制信息
    @Column("cost")
    private Double cost;// 任务耗时
    @Column("node_id")
    private String nodeId; // 节点ID
    @Column("instance_id")
    private Long instanceId; // 实例ID
    @Column("gmt_start")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtStart; // 开始时间
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtFinish; // 结束时间
    @Column("module_type")
    private String moduleType; // 节点类型
    @IgnoreAll
    private List<Object> successes; // 成功的
    @IgnoreAll
    private List<Object> failures; // 失败的
    @IgnoreAll
    private JSONObject outputs;
    @IgnoreAll
    private RequestContext context;
    @IgnoreAll
    private Boolean dicision;
    @IgnoreAll
    private String params;
    @IgnoreAll
    private Long createrId;
    @IgnoreAll
    private String instanceName;

    public void preSave(Long creatorId, Long menderId, Long instanceId, String status) {
        this.setCreatorId(creatorId);
        this.setMenderId(menderId);
        this.instanceId = instanceId;
        this.setStatus(status);
    }

    public static GeneralResult<TaskNode> convert(String content) {
        if (StringUtils.isEmpty(content)) {
            return new GeneralResult<>(false, "参数为空！");
        }
        try {
            TaskNode taskNode = JSONObject.parseObject(content, TaskNode.class);
            return new GeneralResult<TaskNode>(true, taskNode, "success");
        } catch (Exception e) {
            return new GeneralResult<>(false, e.getLocalizedMessage());
        }
    }

}
