package com.bocloud.tse.model;

import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.megatron.common.model.Result;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 目标机器或者HTTP请求的url或者DB的连接URL的执行结果信息封装类
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class HostResult extends Result {

    private Long id;
    private String host;// 这里可以表示远程目标机器，还可以表示远程服务的URL，或者数据库连接的URL
    private Double cost;
    private String rawName;
    private Integer port;
    private String user;
    private String task; // worker当前执行的任务名
    /**
     * worker执行实例
     */
    private String workerInstance;
    private String changes;

    public HostResult() {
        super();
    }

    public HostResult(boolean success, String message, String host, String rawName, Integer port, String user,
        String workerInstance) {
        super(success, message);
        this.host = host;
        this.rawName = rawName;
        this.port = port;
        this.user = user;
        this.workerInstance = workerInstance;
    }

    public static List<HostResult> convertList(String content) throws Exception {
        if (StringUtils.isEmpty(content)) {
            throw new Exception("参数为空！");
        }
        try {
            return JSONArray.parseArray(content, HostResult.class);
        } catch (Exception e) {
            throw new Exception("目标主机任务结果参数异常1", e);
        }
    }

}
