package com.bocloud.tse.model;

import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.megatron.common.enums.OSCategory;

import lombok.Data;

/**
 * Worker请求目标封装类
 */
@Data
public class WorkerHost {

    private Long id;
    private String host;// 主机IP
    private String user;// 操作账号
    private String pasd;// 账号密码
    private Integer port;// SSH端口，默认22
    private String task;// 关联的任务名称
    private String rawName;// 主机名称
    private String category = OSCategory.LINUX.name();// 操作系统类型，默认Linux
    private String connection;
    private Boolean sudo;
    private Long segmentId;
    private Boolean become;
    private String becomeUser;
    private String becomeMethod;
    private String becomePass;
    private boolean free;
    private Long saltId;
    /**
     * salt minion的唯一标识，需要通过这个获取salt执行结果
     */
    private String saltMinionId;

    public WorkerHost() {
        super();
    }

    public WorkerHost(String host, String connection) {
        super();
        this.host = host;
        this.connection = connection;
    }

    public WorkerHost(String host, String user, String pasd, Integer port, OSCategory category) {
        super();
        this.host = host;
        this.user = user;
        this.pasd = pasd;
        this.category = category.name();
        if (null == port || 0 >= port) {
            switch (category) {
                case WINDOWS:
                    this.port = 5985;
                    break;
                case LINUX:
                    this.port = 22;
                    break;
                case UNIX:
                    this.port = 22;
                    break;
                default:
                    this.port = 22;
                    break;
            }
        } else {
            this.port = port;
        }
    }

    public WorkerHost(Long id, String host, String user, String pasd, Integer port, OSCategory category) {
        super();
        this.id = id;
        this.host = host;
        this.user = user;
        this.pasd = pasd;
        this.category = category.name();
        if (null == port || 0 >= port) {
            switch (category) {
                case WINDOWS:
                    this.port = 5985;
                    break;
                case LINUX:
                    this.port = 22;
                    break;
                case UNIX:
                    this.port = 22;
                    break;
                default:
                    this.port = 22;
                    break;
            }
        } else {
            this.port = port;
        }
    }

    public static List<WorkerHost> parseTargets(String content) throws Exception {
        if (StringUtils.isEmpty(content)) {
            throw new Exception("参数为空！");
        } else {
            try {
                return JSONArray.parseArray(content, WorkerHost.class);
            } catch (Exception e) {
                throw new Exception("目标机器参数类型错误！", e);
            }
        }
    }

}
