package com.bocloud.tse.scheduler.core;

import com.megatron.common.model.Result;

import lombok.extern.slf4j.Slf4j;

/**
 * 抽象任务调度器
 */
@Slf4j
public abstract class AbstractScheduler {

    /**
     * 任务数据缓存
     */
    TaskCache taskCache = TaskCache.instance();

    /**
     * 中断指定任务引擎
     */
    public Result suspendTask(String taskId) {
        if (taskCache.hasTaskExecutor(taskId)) {
            taskCache.getTaskExecutor(taskId).suspend(null);
            return new Result(true, "暂停任务命令已下发,待执行中的节点完成后暂停");
        } else {
            log.error("Task 【{}】 does not has TaskEngine in this server!");
            return new Result(false, "TaskEngine Not exist!");
        }
    }

    /**
     * 激活指定任务引擎
     */
    public Result activeTask(String taskId, Long nodeId) {
        if (taskCache.hasTaskExecutor(taskId)) {
            taskCache.getTaskExecutor(taskId).active(nodeId);
            return new Result(true, "success");
        } else {
            log.error("Task 【{}】 does not has TaskEngine in this server!");
            return new Result(false, "TaskEngine Not exist!");
        }
    }

    /**
     * 取消指定任务
     */
    public Result cancelTask(String taskId) {
        if (taskCache.hasTaskExecutor(taskId)) {
            taskCache.getTaskExecutor(taskId).cancel();
            log.info("Cancel Task [{}] success", taskId);
            return new Result(true, "取消任务命令已下发,待执行中的节点完成后取消");
        } else {
            log.error("Task 【{}】 does not has TaskEngine in this server!");
            return new Result(false, "TaskEngine Not exist!");
        }
    }

    /**
     * 销毁任务调度
     */
    abstract void destory();
}
