package com.bocloud.tse.scheduler.listener;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import com.bocloud.tse.model.Job;
import com.bocloud.tse.model.JobResult;
import com.bocloud.tse.model.TaskResult;
import com.bocloud.tse.scheduler.core.TaskCache;
import com.megatron.common.utils.Common;

import lombok.extern.slf4j.Slf4j;

/**
 * TaskNode任务完成处理类
 */
@Slf4j
@Component("jobCompletedListener")
public class JobCompletedListener {

    TaskCache taskCache = TaskCache.instance();

    public void onComplete(JobResult jobResult) {
        if (null == jobResult) {
            return;
        }
        String taskId = null;
        Job job = jobResult.getJob();
        if (null == job || StringUtils.isEmpty(job.getTaskId())) {
            log.error("收到异常消息结果！");
            return;
        }
        taskId = job.getTaskId();
        if (taskCache.hasNodeExecutor(taskId)) {
            TaskResult taskResult = new TaskResult(jobResult.isSuccess(), jobResult.getMsg(), jobResult, taskId);
            taskCache.noticeNodeExecutor(taskId, taskResult);
        } else {
            log.info("Job【{}】不是在本服务器执行，忽略", job.getParam(Common.NAME));
        }
    }

}
