package com.bocloud.tse.scheduler.selector;

import java.util.List;

import com.bocloud.tse.entity.TaskNode;
import com.bocloud.tse.intf.TaskNodeService;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;

/**
 * 任务节点选择器
 */
public class TaskNodeSelector implements NodeSelector<TaskNode> {

    /**
     * 任务实例的ID
     */
    private Long instanceId;

    /**
     * 任务节点服务
     */
    private TaskNodeService nodeService;

    public TaskNodeSelector(Long instanceId, TaskNodeService nodeService) {
        super();
        this.instanceId = instanceId;
        this.nodeService = nodeService;
    }

    @Override
    public GeneralResult<List<TaskNode>> selectNext() {
        List<TaskNode> nodes = nodeService.getReadyNodes(instanceId);
        if (null == nodes) {
            return new GeneralResult<>(false, "获取任务可执行节点异常！");
        }
        return new GeneralResult<List<TaskNode>>(true, nodes, "success");
    }

    @Override
    public GeneralResult<List<TaskNode>> selectLast() {
        List<TaskNode> nodes = nodeService.getLastNodes(instanceId);
        if (null == nodes) {
            return new GeneralResult<>(false, "获取任务最后节点异常！");
        }
        return new GeneralResult<List<TaskNode>>(true, nodes, "success");
    }

    @Override
    public GeneralResult<List<TaskNode>> selectFirst() {
        List<TaskNode> nodes = nodeService.getReadyNodes(instanceId);
        List<TaskNode> freeNodes = nodeService.getFreeNodes(instanceId);
        if (ListTool.isEmpty(nodes) && ListTool.isEmpty(freeNodes)) {
            return new GeneralResult<>(false, "获取任务可执行节点异常！");
        }
        List<TaskNode> listNodes = Lists.newArrayList();
        if (null != nodes) {
            listNodes.addAll(nodes);
        }
        if (null != freeNodes) {
            listNodes.addAll(freeNodes);
        }
        return new GeneralResult<List<TaskNode>>(true, listNodes, "success");
    }

}
