package com.bocloud.tse.scheduler.worker;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public abstract class AbstractWorker {

    String code;
    String nodeId;
    RabbitAdmin rabbitAdmin;
    StringRedisTemplate redisTemplate;
    ServiceFactory serviceFactory;
    LockFactory lockFactory;
    private Integer timeout = Integer.valueOf(10 * 60);

    public AbstractWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        this.code = code;
        this.nodeId = nodeId;
        if (null != timeout && timeout > 0) {
            this.timeout = timeout;
        }
        this.rabbitAdmin = rabbitAdmin;
        this.serviceFactory = serviceFactory;
        this.lockFactory = lockFactory;
        this.redisTemplate = redisTemplate;
    }

    public GeneralResult<?> execute(Object param,String instance) {
        GeneralResult<?> result = null;
        try {
            log.info("init node job [{}]", nodeId);
            //根据具体的worker实现类初始化任务
            init(param);
            log.info("execute node job [{}]", nodeId);
            result = call(instance);
        } catch (Exception e) {
            log.error("worker param illegal!", e);
            result = GeneralResult.FAILED("worker param illegal!!");
        }
        if (result.isFailed()) {
            log.error("node job [{}] result is [{}]", nodeId, JSONObject.toJSONString(result));
        }
        return result;
    }

    protected abstract void init(Object param) throws NodeFormatException;

    protected abstract GeneralResult<?> call(String instanceId);

    public Result buildResult(GeneralResult<?> result) {
        String message = JSONObject.toJSONString(result);
        return new Result(result.isSuccess(), message);
    }

    public Integer getTimeout() {
        return timeout;
    }

    public StringRedisTemplate getRedisTemplate() {
        return redisTemplate;
    }

    public String getCode() {
        return code;
    }

    public String getNodeId() {
        return nodeId;
    }

}