package com.bocloud.tse.scheduler.worker;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.tse.scheduler.worker.locator.WorkerClassLocator;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Repository
public class WorkerFactory {

    private final ServiceFactory serviceFactory;
    private final RabbitAdmin rabbitAdmin;
    private final StringRedisTemplate redisTemplate;
    private final LockFactory lockFactory;
    private WorkerClassLocator classLocator;

    @Autowired
    public WorkerFactory(RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate, ServiceFactory serviceFactory,
        LockFactory lockFactory) {
        this.rabbitAdmin = rabbitAdmin;
        this.redisTemplate = redisTemplate;
        this.serviceFactory = serviceFactory;
        this.lockFactory = lockFactory;
        classLocator = new WorkerClassLocator();
    }

    public AbstractWorker buildWorker(String code, String nodeId, Integer timeout) {
        Class<?> clazz = classLocator.getWorkerClass(code);
        if (null != clazz) {
            try {
                Constructor<?> constructor = clazz.getDeclaredConstructor(String.class, String.class, RabbitAdmin.class,
                    StringRedisTemplate.class, ServiceFactory.class, LockFactory.class, Integer.class);
                try {
                    return (AbstractWorker)constructor.newInstance(code, nodeId, rabbitAdmin, redisTemplate,
                        serviceFactory, lockFactory, timeout);
                } catch (IllegalAccessException e) {
                    log.error("init worker [{}] of [{}] IllegalAccessException!", clazz, code, e);
                }
            } catch (NoSuchMethodException e) {
                log.error("init worker [{}] of [{}] NoSuchMethodException!", clazz, code, e);
            } catch (InstantiationException e) {
                log.error("init worker [{}] of [{}] InstantiationException!", clazz, code, e);
            } catch (InvocationTargetException e) {
                log.error("init worker [{}] of [{}] InvocationTargetException!", clazz, code, e);
            }
        }
        return null;
    }

}