package com.bocloud.tse.scheduler.worker.executor;

import com.alibaba.fastjson.JSONObject;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

/**
 * 通过mq调用Worker服务专用的远程服务
 */
@Slf4j
public class WorkerRemoteMQService extends RemoteService {

    private RabbitAdmin rabbitAdmin;
    private String instance;
    private static final String exchange = "bocloud.topic.exchange";

    public WorkerRemoteMQService(String instance, ServiceParam params, RabbitAdmin rabbitAdmin) {
        super(null, params, true, instance, null);
        this.rabbitAdmin = rabbitAdmin;
        this.instance = instance;
    }

    @Override
    public GeneralResult<?> invoke() {
        try {
            // 发送mq消息到队列
            this.rabbitAdmin.getRabbitTemplate().convertAndSend(exchange,
                "task.distribute." + instance.replace(":", "_"), JSONObject.toJSONString(this.getParam().getParams()));
            JSONObject data = new JSONObject();
            data.put("success", true);
            data.put("message", "任务下发成功");
            return new GeneralResult<>(true, JSONObject.toJSONString(data));
        } catch (Exception e) {
            log.error("distribute task error:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    public static void main(String[] args) {
        System.out.println("IP:PORT".replace(":", "_"));
    }
}
