package com.bocloud.tse.scheduler.worker.executor;

import org.springframework.util.StringUtils;

import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;

import lombok.extern.slf4j.Slf4j;

/**
 * 同步调用Worker服务专用的远程服务
 */
@Slf4j
public class WorkerRemoteService extends RemoteService {

    public WorkerRemoteService(Service service, boolean exist) {
        super(service, exist);
    }

    public WorkerRemoteService(Service service, ServiceParam param, boolean exist, String url, HttpMethod method) {
        super(service, param, exist, url, method);
    }

    @Override
    public GeneralResult<?> invoke() {
        boolean exist = super.isExist();
        String url = super.getUrl();
        HttpMethod method = super.getMethod();
        ServiceParam param = super.getParam();
        PostDataFormat dataFormat = super.getDataFormat();
        Service service = super.getService();

        if ((!exist) || (!StringUtils.hasText(url)) || (method == null)) {
            log.error("Remote service [{}] does not exist!", service.getName());
            return new GeneralResult<>(false, "【" + service.getName() + "】服务未找到！", "404", "远端服务异常，请稍后重试！");
        }
        Result response = null;
        HttpClient client = new HttpClient(10 * 60 * 1000);
        switch (method.getMethod()) {
            case GET:
                response = client.get(param.getHeaders(), param.getParams(), url);
                break;
            case POST:
                client.setDataFormat(dataFormat);
                response = client.post(param.getHeaders(), param.getParams(), url);
                break;
            case DELETE:
                response = client.delete(param.getHeaders(), param.getParams(), url);
                break;
            case PUT:
                response = client.put(param.getHeaders(), param.getParams(), url);
                break;
            case PATCH:
                response = client.patch(param.getHeaders(), param.getParams(), url);
                break;
            case TRACE:
                response = client.trace(param.getHeaders(), param.getParams(), url);
                break;
            default:
                response = client.get(param.getHeaders(), param.getParams(), url);
        }
        return getResult(response);
    }

    private GeneralResult<?> getResult(Result response) {
        return new GeneralResult<>(response.isSuccess(),
            response.isSuccess() ? response.getMessage() : response.getErrorMsg());
    }

}
