package com.bocloud.tse.scheduler.worker.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.enums.TaskParam;
import com.bocloud.tse.model.Job;
import com.bocloud.tse.model.JobResult;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;

/**
 * 抽象任务监听器
 */
public abstract class AbstractTaskListener {
    static final String QUEUE = "realtime.task.queue";
    static final String DATA = TaskParam.TASKDATA.name();
    static final String PJOB = TaskParam.TASKPJOB.name();
    static final String CODE = TaskParam.TASKCODE.name();
    static final String TIME = TaskParam.TIMEOUT.name();
    static final String TYPE = TaskParam.TASKTYPE.name();
    static final String NAME = Common.NAME;
    static final String MODULETYPE = TaskParam.MODULETYPE.name();
    static final String INSTANCE = TaskParam.INSTANCE.name();

    /**
     * 处理接收到任务时的逻辑
     */
    public abstract JobResult onTask(Job job);

    Integer buildTimeout(String time) {
        Integer timeout;
        try {
            timeout = Integer.valueOf(time);
            if (timeout <= 0) {
                timeout = 600;
            }
        } catch (Exception e) {
            timeout = 600;
        }
        return timeout;
    }

    public JobResult buildResult(GeneralResult<?> result, Job job) {
        return new JobResult(result.isSuccess(), JSONObject.toJSONString(result), job);
    }

    public String getInstance(CurrentService service) {
        return service.getInstance().replace(":", "_");
    }

}