package com.bocloud.tse.scheduler.worker.listener;

import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.model.Job;
import com.bocloud.tse.model.JobResult;
import com.bocloud.tse.scheduler.worker.AbstractWorker;
import com.bocloud.tse.scheduler.worker.ParamBuilder;
import com.bocloud.tse.scheduler.worker.WorkerFactory;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

/**
 * 运维类任务监听器
 */
@Slf4j
@Component
public class MaintainTaskListener extends AbstractTaskListener {

    private final WorkerFactory workerFactory;
    private ParamBuilder paramBuilder;
    private final CurrentService currentService;

    @Autowired
    public MaintainTaskListener(WorkerFactory workerFactory, CurrentService currentService) {
        this.workerFactory = workerFactory;
        this.paramBuilder = new ParamBuilder();
        this.currentService = currentService;
    }

    @Override
    public JobResult onTask(Job job) {
        String code = job.getParam(CODE);
        String name = job.getParam(NAME);
        String data = job.getParam(DATA);
        String time = job.getParam(TIME);
        String moduleType = job.getParam(MODULETYPE);
        String instance = job.getParam(INSTANCE);
        log.info("收到运维任务【{}】的节点任务【{}】任务号【{}】,编码为【{}】，节点类型为【{}】", instance, name, job.getTaskId(), code, moduleType);
        try {
            AbstractWorker worker = workerFactory.buildWorker(moduleType, job.getTaskId(), buildTimeout(time));
            JSONObject jsonObject = JSONObject.parseObject(data);
            jsonObject.put("code", code);
            Object param = paramBuilder.buildParam(moduleType, JSONObject.toJSONString(jsonObject));
            if (null == worker) {
                log.error("maintain worker [{}] instance build error!", moduleType);
                throw new NodeFormatException("worker instance build error!");
            }
            if (null == param) {
                log.error("maintain worker [{}] param build error!", moduleType);
                throw new NodeFormatException("worker param build error!");
            }
            return buildResult(worker.execute(param, getInstance(currentService)), job);
        } catch (NodeFormatException e) {
            log.error("maintain worker [{}] param is illegal", moduleType);
            return buildResult(GeneralResult.FAILED(e.getLocalizedMessage()), job);
        } catch (Exception e) {
            log.error("maintain worker [{}] occur unknown exception!", moduleType, e);
            return buildResult(GeneralResult.FAILED(e.getMessage()), job);
        }
    }

}