package com.bocloud.tse.scheduler.worker.listener;

import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.model.Job;
import com.bocloud.tse.model.JobResult;
import com.bocloud.tse.scheduler.worker.AbstractWorker;
import com.bocloud.tse.scheduler.worker.ParamBuilder;
import com.bocloud.tse.scheduler.worker.WorkerFactory;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

/**
 * 软件安装类任务监听器
 */
@Slf4j
@Component
public class SoftwareTaskListener extends AbstractTaskListener {

    private final WorkerFactory workerFactory;
    private ParamBuilder paramBuilder;
    private CurrentService currentService;

    @Autowired
    public SoftwareTaskListener(WorkerFactory workerFactory, CurrentService currentService) {
        this.workerFactory = workerFactory;
        this.paramBuilder = new ParamBuilder();
        this.currentService = currentService;
    }

    @Override
    public JobResult onTask(Job job) {
        // String code = job.getParam(CODE);
        String name = job.getParam(NAME);
        String data = job.getParam(DATA);
        String time = job.getParam(TIME);
        String moduleType = job.getParam(MODULETYPE);
        log.info("receive a software install task! name is [{}] and moduleType is [{}]", name, moduleType);
        try {
            AbstractWorker worker = workerFactory.buildWorker(moduleType, job.getTaskId(), buildTimeout(time));
            Object param = paramBuilder.buildParam(moduleType, data);
            if (null == worker) {
                log.error("worker [{}] instance build error!", moduleType);
                throw new NodeFormatException("worker instance build error!");
            }
            if (null == param) {
                log.error("worker [{}] param build error!", moduleType);
                throw new NodeFormatException("worker param build error!");
            }
            return buildResult(worker.execute(param, getInstance(currentService)), job);
        } catch (NodeFormatException e) {
            log.error("task code [{}] is illegal", moduleType);
            return buildResult(GeneralResult.FAILED(e.getLocalizedMessage()), job);
        } catch (Exception e) {
            log.error("unknown exception!", e);
            return buildResult(GeneralResult.FAILED(e.getMessage()), job);
        }
    }

}