package com.bocloud.tse.scheduler.worker.listener;

import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.tse.model.Job;
import com.bocloud.tse.scheduler.listener.JobCompletedListener;
import com.google.common.util.concurrent.ThreadFactoryBuilder;

/**
 * JOB监听器
 */
@Component("taskJobListener")
public class TaskJobListener implements InitializingBean {

    @Autowired
    private MaintainTaskListener maintainTaskListener;
    @Autowired
    private SoftwareTaskListener softwareTaskListener;
    @Autowired
    private ScheduledTaskListener scheduledTaskListener;
    @Autowired
    private JobCompletedListener jobCompletedListener;
    private ThreadPoolExecutor jobPool = null;

    public void submit(Job job) {
        jobPool.submit(
            new JobTask(maintainTaskListener, softwareTaskListener, scheduledTaskListener, jobCompletedListener, job));
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        int processors = Runtime.getRuntime().availableProcessors();
        jobPool = new ThreadPoolExecutor(processors * 5, Integer.MAX_VALUE, 60L, TimeUnit.SECONDS,
            new SynchronousQueue<Runnable>(), new ThreadFactoryBuilder().build());
        jobPool.allowCoreThreadTimeOut(true);
    }

}
