package com.bocloud.tse.scheduler.worker.locator;

import java.io.File;
import java.io.IOException;
import java.net.JarURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class ClazzLocator {

    public static List<Class<?>> getAllClass(String packageName, String filter) {
        List<Class<?>> list = new ArrayList<>();
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        String path = packageName.replace(".", "/");
        try {
            Enumeration<URL> enumeration = classLoader.getResources(path);
            while (enumeration.hasMoreElements()) {
                URL url = enumeration.nextElement();
                if ("jar".equals(url.getProtocol())) {
                    String jarPath = url.getPath();
                    if (!jarPath.contains(filter)) {
                        continue;
                    }
                    JarFile jarFile = ((JarURLConnection)url.openConnection()).getJarFile();
                    list.addAll(findClassFromJar(jarFile, path));
                } else {
                    list.addAll(findClassFromFile(new File(url.getFile()), packageName));
                }
            }
        } catch (IOException e) {
            log.error("get all class of package {} IOException!", packageName, e);
        }
        return list;
    }

    private static List<Class<?>> findClassFromJar(JarFile jarFile, String path) {
        List<Class<?>> classList = new ArrayList<>();
        String packageName = path.replace(".", "/");
        try {
            Enumeration<JarEntry> entries = jarFile.entries();
            String entryName = null;
            while (entries.hasMoreElements()) {
                JarEntry entry = entries.nextElement();
                if (entry.isDirectory()) {
                    continue;
                }
                entryName = entry.getName();
                if (entryName.charAt(0) == '/') {
                    entryName = entryName.substring(1);
                }
                if (entryName.startsWith("classes/")) {
                    entryName = entryName.substring(8);
                }
                if (!entryName.startsWith(packageName) || !entryName.endsWith(".class")) {
                    continue;
                }
                String className = entryName.substring(0, entryName.length() - 6);
                Class<?> clazz = loadClass(className.replace("/", "."));
                if (null != clazz) {
                    classList.add(clazz);
                }
            }
        } catch (Exception e) {
            log.error("get class from jar {} error!", jarFile.getName(), e);
        } finally {
            if (null != jarFile) {
                try {
                    jarFile.close();
                } catch (Exception e) {
                    log.error("close jar {} error!", jarFile.getName(), e);
                }
            }
        }
        return classList;
    }

    private static List<Class<?>> findClassFromFile(File file, String packageName) {
        List<Class<?>> list = new ArrayList<>();
        if (!file.exists()) {
            return list;
        }
        File[] files = file.listFiles();
        if (null == files || files.length == 0) {
            return list;
        }
        for (File classFile : files) {
            if (classFile.isDirectory()) {
                if (!classFile.getName().contains(".")) {
                    List<Class<?>> arrayList = findClassFromFile(classFile, packageName + "." + classFile.getName());
                    list.addAll(arrayList);
                }
            } else if (classFile.getName().endsWith(".class")) {
                try {
                    // 保存的类文件不需要后缀.class
                    list.add(Class.forName(
                        packageName + '.' + classFile.getName().substring(0, classFile.getName().length() - 6)));
                } catch (ClassNotFoundException e) {
                    log.error("get class of {} occur ClassNotFoundException!", file, e);
                }
            }
        }
        return list;
    }

    private static Class<?> loadClass(String fullClassName) {
        try {
            return Thread.currentThread().getContextClassLoader().loadClass(fullClassName);
        } catch (ClassNotFoundException e) {
            log.error("load class of {} occur ClassNotFoundException!", e);
        }
        return null;
    }

}
