package com.bocloud.tse.scheduler.worker.locator;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.bocloud.tse.core.annotation.Module;
import com.bocloud.tse.core.domain.input.Input;

import lombok.extern.slf4j.Slf4j;

/**
 * 前端输入类定位器，用来对Input的具体实现类基于code进行定位
 */
@Slf4j
public class InputClassLocator {

    private static Map<String, Class<?>> inputClassMap = new ConcurrentHashMap<>();

    private List<Class<?>> getInputClasses(String code) {
        List<Class<?>> allClass = ClazzLocator.getAllClass(Input.class.getPackage().getName(), "input");
        List<Class<?>> fixClass = new ArrayList<>();
        for (Class<?> clazz : allClass) {
            if (clazz.isAnnotationPresent(Module.class)) {
                Module module = (Module)clazz.getAnnotation(Module.class);
                if (module.code().equalsIgnoreCase(code)) {
                    fixClass.add(clazz);
                }
            }
        }
        return fixClass;
    }

    public Class<?> getInputClass(String code) {
        if (inputClassMap.containsKey(code)) {
            return inputClassMap.get(code);
        }
        List<Class<?>> inputClassList = getInputClasses(code);
        if (inputClassList.isEmpty()) {
            return null;
        }
        if (inputClassList.size() > 1) {
            log.error("more than one class found!");
            return null;
        }
        inputClassMap.put(code, inputClassList.get(0));
        return inputClassList.get(0);
    }

}