package com.bocloud.tse.scheduler.worker.maintain;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.maintain.FileArg;
import com.bocloud.tse.core.domain.front.maintain.FileFront;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.BackupInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.BackupModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.maintain.backup")
public class BackupWorker extends MaintainWorker {

    private BackupInput input;

    public BackupWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof BackupInput) {
            input = (BackupInput)param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            // 构建请求参数
            buildParam();
        } else {
            log.error("backup worker param illegel!");
            throw new NodeFormatException("backup worker param illegel!");
        }
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            log.error("backup worker input param is empty!");
            return;
        }
        processBecome(input.getTarget());
        FileFront front = input.getFronts();
        FileArg arg = new FileArg(front.getSource(), null);
        Module<FileArg> module = new BackupModule(arg);
        if (!ListTool.isEmpty(targets)) {
            for (List<WorkerHost> hosts : this.targets) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("module", module);
                param.put("response", "async_playbook");
                param.put("timeout", this.getTimeout());
                paramList.add(param);
            }
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<List<WorkerHost>> list = targetsMap.get(key);
            for (List<WorkerHost> hosts : list) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("module", module);
                param.put("response", "async_playbook");
                param.put("timeout", this.getTimeout());
                if ("noworkergroup".equalsIgnoreCase(key)) {
                    param.put(key, key);
                } else {
                    param.put("groupId", key);
                }
                paramList.add(param);
            }
        }
    }

}