package com.bocloud.tse.scheduler.worker.maintain;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.maintain.HostAllowArg;
import com.bocloud.tse.core.domain.front.maintain.HostAllowFront;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.HostAllowInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.HostAllowModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.maintain.hostallow")
public class HostAllowWorker extends MaintainWorker {

    private HostAllowInput input;
    private HostAllowArg args;

    public HostAllowWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof HostAllowInput) {
            this.input = (HostAllowInput)param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            // 构建请求参数
            buildParam();
        } else {
            log.error("Config hosts allow worker param illegal!");
            throw new NodeFormatException("Worker Param illegal!");
        }
    }

    /**
     * 参数构建
     */
    @Override
    void buildParam() {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        HostAllowFront front = input.getFronts();
        List<String> workers = new ArrayList<>();
        for (JSONObject object : front.getWorkers()) {
            String instance = object.getString("instance");
            if (StringUtils.isEmpty(instance)) {
                continue;
            }
            if (instance.contains(":")) {
                instance = instance.split(":")[0];
            }
            workers.add(instance);
        }
        args = new HostAllowArg(workers, front.getState());
        Module<HostAllowArg> module = new HostAllowModule(args, null);
        for (List<WorkerHost> hosts : this.targets) {
            Map<String, Object> param = MapTools.simpleMap("targets", hosts);
            param.put("module", module);
            param.put("response", "async_playbook");
            param.put("timeout", this.getTimeout());
            paramList.add(param);
        }
    }

}