package com.bocloud.tse.scheduler.worker.maintain;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.maintain.FileArg;
import com.bocloud.tse.core.domain.front.maintain.FileFront;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.RecoveryInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.RecoveryModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.maintain.recovery")
public class RecoveryWorker extends MaintainWorker {
	private RecoveryInput input;

	public RecoveryWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                          ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
		super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
	}

	@Override
	protected void init(Object param) throws NodeFormatException {
		if (param instanceof RecoveryInput) {
			input = (RecoveryInput) param;
			Target target = input.getTarget();
			// 目标主机切片
			burst(target);
			// 构建请求参数
			buildParam();
		} else {
			log.error("recovery worker param illegel!");
			throw new NodeFormatException("recovery worker param illegel!");
		}
	}

	@Override
	void buildParam() throws NodeFormatException {
		if (null == input) {
			log.error("recovery worker input param is empty!");
			return;
		}
		processBecome(input.getTarget());
		FileFront front = input.getFronts();
		List<String> sources = JSONArray.parseArray(front.getSource(), String.class);
		int index = 0;
		for (List<WorkerHost> hosts : this.targets) {
			FileArg arg = new FileArg(sources.get(index), front.getTarget());
			Module<FileArg> module = new RecoveryModule(arg);
			Map<String, Object> param = MapTools.simpleMap("targets", hosts);
			param.put("module", module);
			param.put("response", "async_playbook");
			param.put("timeout", this.getTimeout());
			paramList.add(param);
			index++;
		}
		Iterator<String> iterator = targetsMap.keySet().iterator();
		while (iterator.hasNext()) {
			String key = iterator.next();
			List<List<WorkerHost>> list = targetsMap.get(key);
			for (List<WorkerHost> hosts : list) {
				FileArg arg = new FileArg(sources.get(index), front.getTarget());
				Module<FileArg> module = new RecoveryModule(arg);
				Map<String, Object> param = MapTools.simpleMap("targets", hosts);
				param.put("module", module);
				param.put("response", "async_playbook");
				param.put("timeout", this.getTimeout());
				if ("noworkergroup".equalsIgnoreCase(key)) {
					param.put(key, key);
				} else {
					param.put("groupId", key);
				}
				paramList.add(param);
				index++;
			}
		}
	}
}