package com.bocloud.tse.scheduler.worker.maintain;

import com.bocloud.tse.constant.ExecuteTypeConstant;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.maintain.ScriptLocalArg;
import com.bocloud.tse.core.domain.front.maintain.ScriptLocalFront;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.ScriptLocalInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.ScriptLocalModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

@Slf4j
@Worker(name = "task.maintain.script.local")
public class ScriptLocalWorker extends MaintainWorker {

    private ScriptLocalInput input;

    /**
     * 构造方法
     *
     * @param code
     * @param rabbitAdmin
     * @param serviceFactory
     * @param timeout
     */
    public ScriptLocalWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                             ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof ScriptLocalInput) {
            this.input = (ScriptLocalInput) param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            // 构建请求参数
            buildParam();
        } else {
            log.error("Script worker param illegal!");
            throw new NodeFormatException("Worker Param illegal!");
        }
    }

    /**
     * 参数构建
     */
    @Override
    void buildParam() {
        if (null == input) {
            return;
        }
        //task.maintain.script
        String code = input.getCode();
        if (StringUtils.isBlank(code)) {
            code = "";
        }
        // 处理become相关参数
        processBecome(input.getTarget());
        //构建参数
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        ScriptLocalFront front = input.getFronts();
        ScriptLocalArg arg = new ScriptLocalArg(front.getContent(),front.getCategory());
        Module<ScriptLocalArg> module = new ScriptLocalModule(arg, null);
        for (List<WorkerHost> hosts : this.targets) {
            Map<String, Object> param = MapTools.simpleMap("targets", hosts);
            param.put("module", module);
            param.put("response", "async_playbook");
            param.put("timeout", this.getTimeout());
            paramList.add(param);
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<List<WorkerHost>> list = targetsMap.get(key);
            for (List<WorkerHost> hosts : list) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("module", module);
                param.put("response", "async_playbook");
                param.put("timeout", this.getTimeout());
                param.put("groupId", key);
                paramList.add(param);
            }
        }
    }
}