package com.bocloud.tse.scheduler.worker.maintain;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Matcher;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.maintain.ScriptsArg;
import com.bocloud.tse.core.domain.front.maintain.ScriptsFront;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.ScriptsInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.ScriptsModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.maintain.scripts")
public class ScriptsWorker extends MaintainWorker {
    private ScriptsInput input;

    public ScriptsWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            log.error("upload worker input param is empty!");
            return;
        }
        String code = input.getCode();
        if (StringUtils.isBlank(code)) {
            code = "";
        }
        //处理become相关参数
        processBecome(input.getTarget());
        if (code.contains("task.maintain.custom")) {
            buildUnInternalNodeParam();
        } else {
            buildInternalNodeParam();
        }

    }

    @Override
    public void init(Object param) throws NodeFormatException {
        if (param instanceof ScriptsInput) {
            input = (ScriptsInput)param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            // 构建请求参数
            buildParam();
        } else {
            log.error("run sripts worker param illegel!");
            throw new NodeFormatException("run sripts worker param illegel!");
        }
    }

    private void buildInternalNodeParam() {
        ScriptsFront front = input.getFronts();
        List<Map<String, Object>> scripts = front.getScripts();
        if (!ListTool.isEmpty(scripts)) {
            for (Map<String, Object> script : scripts) {
                Iterator<Entry<String, Object>> iterator = script.entrySet().iterator();
                while (iterator.hasNext()) {
                    Entry<String, Object> entry = iterator.next();
                    if (null == entry.getValue() || StringUtils.isEmpty(entry.getValue().toString())) {
                        iterator.remove();
                    }
                }
                if (!script.containsKey("params")) {
                    continue;
                }
                String content = script.get("content").toString();
                List<JSONObject> params =
                    JSONObject.parseArray(JSONObject.toJSONString(script.get("params")), JSONObject.class);
                for (JSONObject param : params) {
                    content =
                        content.replaceAll("\\$\\{\\{" + param.getString("key") + "\\}\\}", Matcher.quoteReplacement(param.getString("value")));
                }
                script.put("content", content);
                script.remove("params");
            }
        }
        ScriptsArg arg = new ScriptsArg(scripts);
        Module<ScriptsArg> module = new ScriptsModule(arg);
        if (!ListTool.isEmpty(targets)) {
            for (List<WorkerHost> hosts : this.targets) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                paramList.add(param);
            }
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<List<WorkerHost>> list = targetsMap.get(key);
            for (List<WorkerHost> hosts : list) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                if ("noworkergroup".equalsIgnoreCase(key)) {
                    param.put(key, key);
                } else {
                    param.put("groupId", key);
                }
                paramList.add(param);
            }
        }
    }

    private void buildUnInternalNodeParam() {
        Encryptor encryptor = new AESEncryptor();
        ScriptsFront front = input.getFronts();
        List<Map<String, Object>> scripts = front.getScripts();
        List<JSONObject> params = JSONObject.parseArray(JSONObject.toJSONString(front.getParams()), JSONObject.class);
        if (!ListTool.isEmpty(scripts) && !ListTool.isEmpty(params)) {
            for (Map<String, Object> script : scripts) {
                Iterator<Entry<String, Object>> iterator = script.entrySet().iterator();
                StringBuffer sb = new StringBuffer();
                while (iterator.hasNext()) {
                    Entry<String, Object> entry = iterator.next();
                    if (null == entry.getValue() || StringUtils.isEmpty(entry.getValue().toString())) {
                        iterator.remove();
                    }
                }

                String content = script.get("content").toString();
                for (JSONObject param : params) {
                    // 处理脚本参数
                    Boolean internal = param.getBoolean("internal") == null ? false : param.getBoolean("internal");
                    if (internal) {
                        content = content.replaceAll("\\$\\{\\{" + param.getString("key") + "\\}\\}",
                            Matcher.quoteReplacement(param.getString("value")));
                    } else if (!internal) {
                        String value = "";
                        if ("ENCRYPT".equalsIgnoreCase(param.getString("type"))) {
                            // 加密参数,需要解密
                            value = encryptor.decrypt(param.getString("value"), null);
                        } else {
                            value = param.getString("value");
                        }
                        sb.append(" ").append(value);
                    }

                }
                content = content + sb.toString();
                script.put("content", content);
                script.remove("params");
            }
        }
        ScriptsArg arg = new ScriptsArg(scripts);
        Module<ScriptsArg> module = new ScriptsModule(arg);
        if (!ListTool.isEmpty(targets)) {
            for (List<WorkerHost> hosts : this.targets) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                paramList.add(param);
            }
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<List<WorkerHost>> list = targetsMap.get(key);
            for (List<WorkerHost> hosts : list) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                if ("noworkergroup".equalsIgnoreCase(key)) {
                    param.put(key, key);
                } else {
                    param.put("groupId", key);
                }
                paramList.add(param);
            }
        }
    }

}