package com.bocloud.tse.scheduler.worker.maintain;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.maintain.ServiceArg;
import com.bocloud.tse.core.domain.front.maintain.ServiceFront;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.ServiceInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.ServiceModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.maintain.service")
public class ServiceWorker extends MaintainWorker {

    private ServiceInput input;
    private ServiceArg arg;

    public ServiceWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                         ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            log.error("service worker input param is empty!");
            return;
        }
        processBecome(input.getTarget());
        ServiceFront front = input.getFronts();
        String state = StringUtils.isEmpty(front.getState()) ? null : front.getState();
        Boolean enabled = null == front.getEnabled() ? null : front.getEnabled();
        String arguments = StringUtils.isEmpty(front.getArguments()) ? null : front.getArguments();
        String startMode = StringUtils.isEmpty(front.getStartMode()) ? null : front.getStartMode();
        arg = new ServiceArg(front.getName(), state, enabled, arguments, startMode);
        Module<ServiceArg> module = new ServiceModule(arg);
        if (!ListTool.isEmpty(targets)) {
            for (List<WorkerHost> hosts : this.targets) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                paramList.add(param);
            }
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<List<WorkerHost>> list = targetsMap.get(key);
            for (List<WorkerHost> hosts : list) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                if ("noworkergroup".equalsIgnoreCase(key)) {
                    param.put(key, key);
                } else {
                    param.put("groupId", key);
                }
                paramList.add(param);
            }
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof ServiceInput) {
            input = (ServiceInput) param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            buildParam();
        } else {
            log.error("service operate task param illegel!!");
            throw new NodeFormatException("service operate task param illegel!!");
        }
    }
}