package com.bocloud.tse.scheduler.worker.maintain;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.Option;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.TemplateInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.TemplateModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

@Slf4j
@Worker(name = "task.maintain.template")
public class TemplateWorker extends MaintainWorker {

    private static Logger logger = LoggerFactory.getLogger(TemplateWorker.class);
    private TemplateInput input;

    public TemplateWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                          ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof TemplateInput) {
            this.input = (TemplateInput) param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            // 构建请求参数
            buildParam();
        } else {
            logger.error("template worker param illegal!");
            throw new NodeFormatException("Worker Param illegal!");
        }
    }

    /**
     * 参数构建
     */
    @Override
    void buildParam() {
        if (null == input) {
            return;
        }
        //处理become相关参数
        processBecome(input.getTarget());

        Target target = input.getTarget();

        Option options = new Option(null, target.getBecome(), target.getBecomeUser(), target.getBecomePass());
        Module<JSONObject> module =
                new TemplateModule(input.getFronts().getName(), input.getFronts().getArgs(), options);
        if (!ListTool.isEmpty(targets)) {
            for (List<WorkerHost> hosts : this.targets) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("module", module);
                param.put("response", "async_playbook");
                param.put("timeout", this.getTimeout());
                paramList.add(param);
            }
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<List<WorkerHost>> list = targetsMap.get(key);
            for (List<WorkerHost> hosts : list) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("module", module);
                param.put("response", "async_playbook");
                param.put("timeout", this.getTimeout());
                if ("noworkergroup".equalsIgnoreCase(key)) {
                    param.put(key, key);
                } else {
                    param.put("groupId", key);
                }
                paramList.add(param);
            }
        }
    }

}