package com.bocloud.tse.scheduler.worker.maintain;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.maintain.UserArg;
import com.bocloud.tse.core.domain.front.maintain.UserFront;
import com.bocloud.tse.core.domain.input.Target;
import com.bocloud.tse.core.domain.input.maintain.UserInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.core.module.maintain.UserModule;
import com.bocloud.tse.model.WorkerHost;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.maintain.user")
public class UserWorker extends MaintainWorker {

    private UserInput input;
    private UserArg arg;

    public UserWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            log.error("user worker input param is empty!");
            return;
        }
        processBecome(input.getTarget());
        UserFront front = input.getFronts();
        Map<String, Object> sudoMap = Maps.newHashMap();
        Map<String, Object> sudo = front.getSudo();
        if (null != sudo && !sudo.isEmpty()) {
            for (String key : sudo.keySet()) {
                if (null != sudo.get(key)) {
                    sudoMap.put(key, sudo.get(key));
                }
            }
        }
        List<Map<String, Object>> userList = Lists.newArrayList();
        List<Map<String, Object>> users = front.getUsers();
        if (!ListTool.isEmpty(users)) {
            for (Map<String, Object> user : users) {
                Map<String, Object> userMap = Maps.newHashMap();
                for (String key : user.keySet()) {
                    if (null == user.get(key)) {
                        continue;
                    }
                    userMap.put(key, user.get(key));
                    if ("expires".equals(key)) {
                        try {
                            SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd");
                            df.setLenient(false);
                            df.setTimeZone(TimeZone.getTimeZone("UTC"));
                            Date date = df.parse(String.valueOf(user.get(key)));
                            userMap.put(key, date.getTime() / 1000);
                        } catch (ParseException e) {
                        }
                    }
                }
                if (!userMap.isEmpty()) {
                    userList.add(userMap);
                }
            }
        }
        userList = userList.isEmpty() ? null : userList;
        sudoMap = sudoMap.isEmpty() ? null : sudoMap;
        arg = new UserArg(front.getType(), sudoMap, userList);
        Module<UserArg> module = new UserModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            for (List<WorkerHost> hosts : this.targets) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                paramList.add(param);
            }
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<List<WorkerHost>> list = targetsMap.get(key);
            for (List<WorkerHost> hosts : list) {
                Map<String, Object> param = MapTools.simpleMap("targets", hosts);
                param.put("timeout", this.getTimeout());
                param.put("response", "async_playbook");
                param.put("module", module);
                if ("noworkergroup".equalsIgnoreCase(key)) {
                    param.put(key, key);
                } else {
                    param.put("groupId", key);
                }
                paramList.add(param);
            }
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof UserInput) {
            input = (UserInput)param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            buildParam();
        } else {
            log.error("user operate task param illegel!!");
            throw new NodeFormatException("user operate task param illegel!!");
        }
    }

}