package com.bocloud.tse.scheduler.worker.software;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.constant.CommonConstant;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.ActivemqArg;
import com.bocloud.tse.core.domain.front.software.ActivemqFront;
import com.bocloud.tse.core.domain.input.software.ActivemqInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.ActivemqModule;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.activemq")
public class ActivemqWorker extends SoftwareWorker {

    private ActivemqInput input;
    private ActivemqArg arg;

    public ActivemqWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        ActivemqFront front = input.getFronts();
        arg = getArg(front);
        module = new ActivemqModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }

    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof ActivemqInput) {
            input = (ActivemqInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("activemq install task param illegel!!");
            throw new NodeFormatException("activemq install task param illegel!!");
        }
    }

    private ActivemqArg getArg(ActivemqFront front) {
        Encryptor encryptor = new AESEncryptor();
        List<Map<String, Object>> users = front.getActivemqAdminUser();
        if (ListTool.isEmpty(users)) {
            users = new ArrayList<>();
        }
        String password = CommonConstant.SOFTPSWD;
        try {
            password = encryptor.encrypt(password, null);
        } catch (Exception e1) {
        }
        Map<String, Object> manager = new HashMap<>();
        manager.put("username", CommonConstant.SOFTUSER);
        manager.put("password", password);
        users.add(manager);
        return new ActivemqArg(front.getVersion(), users);
    }

}