package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import com.bocloud.tse.constant.ExecuteTypeConstant;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.ApacheArg;
import com.bocloud.tse.core.domain.front.software.ApacheFront;
import com.bocloud.tse.core.domain.input.software.ApacheInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.ApacheModule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.apache")
public class ApacheWorker extends SoftwareWorker {
    private ApacheInput input;
    private ApacheArg arg;

    public ApacheWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        String executeType =  input.getTarget().getExecuteType();
        processBecome(input.getTarget());
        ApacheFront front = input.getFronts();
        arg = new ApacheArg(front.getSoftwareConfig(), front.getApacheVhosts());
        module = new ApacheModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            param.put("executeType", executeType);
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            param.put("executeType", executeType);
            if (ExecuteTypeConstant.SALTSTACK.equalsIgnoreCase(executeType)) {
                param.put("saltId", key);
            } else {
                if ("noworkergroup".equalsIgnoreCase(key)) {
                    param.put(key, key);
                } else {
                    param.put("groupId", key);
                }
            }
            paramList.add(param);
        }

    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof ApacheInput) {
            input = (ApacheInput) param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getExecuteType(), input.getTarget().getData());
            buildParam();
        } else {
            log.error("apache install task param illegel!!");
            throw new NodeFormatException("apache install task param illegel!!");
        }
    }
}