package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.ExporterArg;
import com.bocloud.tse.core.domain.front.software.ExporterFront;
import com.bocloud.tse.core.domain.input.software.ExporterInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.ExporterModule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.exporter")
public class ExporterWorker extends SoftwareWorker {

    private ExporterInput input;
    private ExporterArg arg;

    public ExporterWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                          ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        ExporterFront front = input.getFronts();
        arg = new ExporterArg(front.getVersion(), front.getPath());
        module = new ExporterModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", this.getTimeout());
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof ExporterInput) {
            input = (ExporterInput) param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("node exporter install task param illegel!!");
            throw new NodeFormatException("node exporter install task param illegel!!");
        }
    }
}