package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import com.bocloud.tse.constant.ExecuteTypeConstant;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.JenkinsArg;
import com.bocloud.tse.core.domain.front.software.JenkinsFront;
import com.bocloud.tse.core.domain.input.software.JenkinsInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.JenkinsModule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.jenkins")
public class JenkinsWorker extends SoftwareWorker {

    JenkinsInput input;
    JenkinsArg arg;

    public JenkinsWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        JenkinsFront front = input.getFronts();
        arg =
            new JenkinsArg(front.getSoftwareConfig(), front.getJenkinsAdminUsername(), front.getJenkinsAdminPassword());
        module = new JenkinsModule(arg, null);
        String executeType = this.input.getTarget().getExecuteType();
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            param.put("executeType", executeType);
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            param.put("executeType", executeType);
            if (ExecuteTypeConstant.SALTSTACK.equalsIgnoreCase(executeType)) {
                this.module.setName("install_jenkins");
                param.put("saltId", key);
            } else if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof JenkinsInput) {
            input = (JenkinsInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getExecuteType(), input.getTarget().getData());
            buildParam();
        } else {
            log.error("jenkins install task param illegal !!");
            throw new NodeFormatException("jenkins install task param illegal !!");
        }
    }
}