package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.K8sArg;
import com.bocloud.tse.core.domain.front.software.K8sFront;
import com.bocloud.tse.core.domain.group.K8sGroup;
import com.bocloud.tse.core.domain.input.software.K8sInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.K8sModule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.k8s")
public class K8sWorker extends SoftwareWorker {
    private K8sGroup group;
    private K8sInput input;
    private K8sArg arg;

    public K8sWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                     ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        K8sFront front = input.getFronts();
        this.group = new K8sGroup(front);
        this.arg = getArg(front);
        this.module = new K8sModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("groups", this.group.getGroups());
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("groups", this.group.getGroups());
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof K8sInput) {
            this.input = (K8sInput) param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("k8s install task param illegel!!");
            throw new NodeFormatException("k8s install task param illegel!!");
        }
    }

    private K8sArg getArg(K8sFront front) {
        return new K8sArg(front.getVersion(), front.isClusterInstall(), front.getSoftwareConfig(),
                front.getContainerConfig(), front.getK8sConfig(), front.getNetworkConfig(), front.getStorageConfig(),
                front.getHarborConfig());
    }
}