package com.bocloud.tse.scheduler.worker.software;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.KafkaArg;
import com.bocloud.tse.core.domain.front.software.KafkaFront;
import com.bocloud.tse.core.domain.group.KafkaGroup;
import com.bocloud.tse.core.domain.input.software.KafkaInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.KafkaModule;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.kafka")
public class KafkaWorker extends SoftwareWorker {
    private KafkaInput input;
    private KafkaArg arg;
    private KafkaGroup group;

    public KafkaWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        KafkaFront front = input.getFronts();
        this.group = new KafkaGroup(front);
        this.arg = getArg(front);
        module = new KafkaModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("groups", this.group.getGroups());
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("groups", this.group.getGroups());
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof KafkaInput) {
            input = (KafkaInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("kafka install task param illegel!!");
            throw new NodeFormatException("kafka install task param illegel!!");
        }
    }

    private KafkaArg getArg(KafkaFront front) {
        Encryptor encryptor = new AESEncryptor();
        Map<String, Object> config = front.getAuthConfig();
        try {
            if (null != config && !config.isEmpty()) {
                config.put("zk_client_password", encryptor.encrypt(config.get("zk_client_password").toString(), null));
                config.put("kafka_admin_password",
                    encryptor.encrypt(config.get("kafka_admin_password").toString(), null));
                List<JSONObject> users =
                    JSONObject.parseArray(JSONObject.toJSONString(config.get("kafka_users")), JSONObject.class);
                if (!ListTool.isEmpty(users)) {
                    for (JSONObject user : users) {
                        user.put("password", encryptor.encrypt(user.get("password").toString(), null));
                    }
                    config.put("kafka_users", users);
                }
            }
        } catch (Exception e) {
        }
        Map<String, Object> kafkaServer = null;
        if (this.group.isNetworks()) {
            kafkaServer = new HashMap<>();
            kafkaServer.put("listener_security_protocol_map", "INSIDE:PLAINTEXT,OUTSIDE:PLAINTEXT");
            kafkaServer.put("inter_broker_listener_name", "INSIDE");
        }
        return new KafkaArg(front.getVersion(), front.isClusterInstall(), front.getSoftwareConfig(),
            front.getZookeeper(), front.getManageConfig(), front.getJmxExporterConfig(), front.isInstallJmxExporter(),
            front.getMntDisks(), true, config, kafkaServer, true);
    }

}