package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.constant.CommonConstant;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.MongodbArg;
import com.bocloud.tse.core.domain.front.software.MongodbFront;
import com.bocloud.tse.core.domain.group.MongoGroup;
import com.bocloud.tse.core.domain.input.software.MongodbInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.MongoModule;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.mongodb")
public class MongodbWorker extends SoftwareWorker {
    private MongodbInput input;
    private MongoGroup group;
    private MongodbArg arg;

    public MongodbWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        MongodbFront front = input.getFronts();
        this.group = new MongoGroup(front);
        this.arg = getArg(front);
        this.module = new MongoModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("groups", this.group.getGroups());
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", this.getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("groups", this.group.getGroups());
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof MongodbInput) {
            this.input = (MongodbInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("mongodb install task param illegel!!");
            throw new NodeFormatException("mongodb install task param illegel!!");
        }
    }

    private MongodbArg getArg(MongodbFront front) {
        Encryptor encryptor = new AESEncryptor();
        Map<String, Object> config = front.getManageConfig();
        config.put("mongodb_replication_replset", front.isClusterInstall() ? "zeus" : "");
        try {
            if (!config.containsKey("mongodb_bocloud_admin_password")) {
                config.put("mongodb_bocloud_admin_name", CommonConstant.SOFTUSER);
                config.put("mongodb_bocloud_admin_password", encryptor.encrypt(CommonConstant.SOFTPSWD, null));
            }
        } catch (Exception e) {
        }
        return new MongodbArg(front.getVersion(), front.isClusterInstall(), front.getSoftwareConfig(), config,
            front.getMongodbUsers(), front.getMntDisks(), true);
    }
}