package com.bocloud.tse.scheduler.worker.software;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.constant.CommonConstant;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.MysqlArg;
import com.bocloud.tse.core.domain.front.software.MysqlFront;
import com.bocloud.tse.core.domain.group.MysqlGroup;
import com.bocloud.tse.core.domain.input.software.MysqlInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.MysqlModule;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.mysql")
public class MysqlWorker extends SoftwareWorker {
    private MysqlGroup group;
    private MysqlInput input;
    private MysqlArg arg;

    public MysqlWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        MysqlFront front = input.getFronts();
        this.group = new MysqlGroup(front);
        this.arg = getArg(front);
        this.module = new MysqlModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targets));
            param.put("module", module);
            param.put("groups", this.group.getGroups());
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", this.getTimeout());
            param.put("response", "async_playbook");
            // param.put("log_tradition", true); 默认false，日志格式为worker格式
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("groups", this.group.getGroups());
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof MysqlInput) {
            this.input = (MysqlInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("mysql install task param illegel!!");
            throw new NodeFormatException("mysql install task param illegel!!");
        }
    }

    private MysqlArg getArg(MysqlFront front) {
        Encryptor encryptor = new AESEncryptor();
        Map<String, Object> config = front.getSoftwareConfig();
        Map<String, Object> replicatUser = front.getMysqlReplicationUser();
        List<Map<String, Object>> users = front.getMysqlUsers();
        if (ListTool.isEmpty(users)) {
            users = new ArrayList<>();
        }
        Map<String, Object> manager = new HashMap<>();
        manager.put("name", CommonConstant.SOFTUSER);
        String password = CommonConstant.SOFTPSWD;
        try {
            password = encryptor.encrypt(password, null);
        } catch (Exception e1) {
        }
        manager.put("password", password);
        manager.put("host", "%");
        manager.put("priv", "*.*:ALL PRIVILEGES,GRANT");
        users.add(manager);
        String groupName = null;
        if ("mgr".equalsIgnoreCase(front.getClusterType())) {
            groupName = UUID.randomUUID().toString();
        }
        return new MysqlArg(front.getVersion(), front.isClusterInstall(), front.getClusterType(), groupName,
            front.getKeepalivedVirtualRouterId(), front.getClusterVip(), front.getMysqlVipNetmask(),
            front.getMysqlReplicationMaster(), config, front.getMysqlDatabases(), users, replicatUser,
            front.getMntDisks(), true);
    }
}