package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.OracleArg;
import com.bocloud.tse.core.domain.front.software.OracleFront;
import com.bocloud.tse.core.domain.input.software.OracleInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.OracleModule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.oracle")
public class OracleWorker extends SoftwareWorker {
    private OracleInput input;
    private OracleArg arg;

    public OracleWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        OracleFront front = input.getFronts();
        arg = getArg(front);
        module = new OracleModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("groups", front.getGroups());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("groups", front.getGroups());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof OracleInput) {
            input = (OracleInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("oracle install task param illegel!!");
            throw new NodeFormatException("oracle server install task param illegel!!");
        }
    }

    private OracleArg getArg(OracleFront front) {
        return new OracleArg(front.getVersion(), front.isClusterInstall(), front.getSoftwareConfig(),
            front.getManageConfig(), front.getOracleDatabases(), front.getAsmDiskGroups(), front.getHostFsLayout());
    }
}