package com.bocloud.tse.scheduler.worker.software;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.constant.CommonConstant;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.PostgresqlArg;
import com.bocloud.tse.core.domain.front.software.PostgresqlFront;
import com.bocloud.tse.core.domain.group.PostgresqlGroup;
import com.bocloud.tse.core.domain.input.software.PostgresqlInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.PostgresqlModule;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.postgresql")
public class PostgresqlWorker extends SoftwareWorker {

    private PostgresqlGroup group;
    private PostgresqlInput input;
    private PostgresqlArg arg;

    public PostgresqlWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        PostgresqlFront front = input.getFronts();
        if (front.isClusterInstall()) {
            this.group = new PostgresqlGroup(front);
        }
        this.arg = getArg(front);
        this.module = new PostgresqlModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            if (null != this.group) {
                param.put("groups", this.group.getGroups());
            }
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            if (null != this.group) {
                param.put("groups", this.group.getGroups());
            }
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof PostgresqlInput) {
            this.input = (PostgresqlInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("postgresql install task param illegel!!");
            throw new NodeFormatException("postgresql install task param illegel!!");
        }
    }

    private PostgresqlArg getArg(PostgresqlFront front) {
        Encryptor encryptor = new AESEncryptor();
        List<Map<String, Object>> users = front.getPostgresqlUsers();
        List<Map<String, Object>> privileges = front.getPostgresqlPrivileges();
        if (ListTool.isEmpty(users)) {
            users = new ArrayList<>();
        }
        if (ListTool.isEmpty(privileges)) {
            privileges = new ArrayList<>();
        }
        String password = CommonConstant.SOFTPSWD;
        try {
            password = encryptor.encrypt(password, null);
        } catch (Exception e) {
        }
        Map<String, Object> manager = new HashMap<>();
        manager.put("name", CommonConstant.SOFTUSER);
        manager.put("pass", password);
        users.add(manager);
        Map<String, Object> privilege = new HashMap<>();
        privilege.put("name", CommonConstant.SOFTUSER);
        privilege.put("db", "postgres");
        privilege.put("priv", "ALL");
        privilege.put("role_attr_flags", "SUPERUSER");
        privileges.add(privilege);
        return new PostgresqlArg(front.getVersion(), front.isClusterInstall(), front.getSoftwareConfig(),
            front.getPostgresqlDatabases(), users, privileges, front.getPostgresqlSchemas(),
            front.getPostgresqlExtensions(), front.getMntDisks(), front.getEtcdConfig(), front.getClusterConfig(), true,
            CommonConstant.SOFTUSER, password);
    }

}