package com.bocloud.tse.scheduler.worker.software;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.constant.CommonConstant;
import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.RabbitmqArg;
import com.bocloud.tse.core.domain.front.software.RabbitmqFront;
import com.bocloud.tse.core.domain.group.RabbitmqGroup;
import com.bocloud.tse.core.domain.input.software.RabbitmqInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.RabbitModule;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.rabbitmq")
public class RabbitmqWorker extends SoftwareWorker {
    private RabbitmqInput input;
    private RabbitmqArg arg;

    public RabbitmqWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        arg = getArg(input.getFronts());
        module = new RabbitModule(arg, null);

        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            List<WorkerHost> hosts = getHosts(targets);
            param.put("targets", hosts);
            param.put("module", this.module);
            param.put("response", "async_playbook");
            if (input.getFronts().isClusterInstall()) {
                param.put("groups", new RabbitmqGroup(input.getFronts(), hosts).getGroups());
            }
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            List<WorkerHost> hosts = getHosts(targetsMap.get(key));
            param.put("targets", hosts);
            param.put("module", this.module);
            param.put("response", "async_playbook");
            if (input.getFronts().isClusterInstall()) {
                param.put("groups", new RabbitmqGroup(input.getFronts(), hosts).getGroups());
            }
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof RabbitmqInput) {
            input = (RabbitmqInput)param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("rabbitmq install task param illegel!!");
            throw new NodeFormatException("rabbitmq install task param illegel!!");
        }
    }

    private RabbitmqArg getArg(RabbitmqFront front) {
        Encryptor encryptor = new AESEncryptor();
        List<Map<String, Object>> users = front.getRabbitmqUsers();
        String password = CommonConstant.SOFTPSWD;
        try {
            password = encryptor.encrypt(password, null);
        } catch (Exception e) {
        }
        Map<String, Object> manager = new HashMap<>();
        manager.put("user", CommonConstant.SOFTUSER);
        manager.put("password", password);
        manager.put("tags", "administrator");
        manager.put("configure_priv", ".*");
        manager.put("read_priv", ".*");
        manager.put("write_priv", ".*");
        manager.put("vhost", "/");
        users.add(manager);
        return new RabbitmqArg(front.getVersion(), front.isClusterInstall(), users, front.getRabbitmqVhosts(),
            front.getRabbitmqPlugins(), front.getMntDisks(), true, CommonConstant.SOFTUSER, password);
    }

}