package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.RedisArg;
import com.bocloud.tse.core.domain.front.software.RedisFront;
import com.bocloud.tse.core.domain.group.RedisGroup;
import com.bocloud.tse.core.domain.input.software.RedisInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.RedisModule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.redis")
public class RedisWorker extends SoftwareWorker {
    private RedisInput input;
    private RedisGroup group;
    private RedisArg arg;

    public RedisWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                       ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        RedisFront front = input.getFronts();
        this.group = new RedisGroup(front);
        this.arg = getArg(front);
        this.module = new RedisModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("groups", this.group.getGroups());
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("groups", this.group.getGroups());
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("response", "async_playbook");
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof RedisInput) {
            this.input = (RedisInput) param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("redis install task param illegel!!");
            throw new NodeFormatException("redis install task param illegel!!");
        }
    }

    private RedisArg getArg(RedisFront front) {
        Map<String, Object> manageConfig = front.getManageConfig();
        if (null != manageConfig) {
            if (manageConfig.containsKey("redis_password")) {
                if (null == manageConfig.get("redis_password")
                        || "".equals(manageConfig.get("redis_password".toString()))) {
                    manageConfig.remove("redis_password");
                }
            }
        }
        return new RedisArg(front.getVersion(), front.isClusterInstall(), front.getSoftwareConfig(), manageConfig,
                front.getMntDisks(), true, front.getRedisMasterNodes(), front.getRedisSlaveNodes());
    }
}