package com.bocloud.tse.scheduler.worker.software;

import java.util.Iterator;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.core.annotation.Worker;
import com.bocloud.tse.core.domain.args.software.RocketmqArg;
import com.bocloud.tse.core.domain.front.software.RocketmqFront;
import com.bocloud.tse.core.domain.group.RocketmqGroup;
import com.bocloud.tse.core.domain.input.software.RocketmqInput;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.software.RocketmqModule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Worker(name = "task.software.rocketmq")
public class RocketmqWorker extends SoftwareWorker {
    private RocketmqGroup group;
    private RocketmqInput input;
    private RocketmqArg arg;

    public RocketmqWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
                          ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
    }

    @Override
    void buildParam() throws NodeFormatException {
        if (null == input) {
            return;
        }
        processBecome(input.getTarget());
        RocketmqFront front = input.getFronts();
        this.group = new RocketmqGroup(front);
        this.arg = new RocketmqArg(front.getVersion(), front.isClusterInstall(), front.getSoftwareConfig(),
                front.getClusterName(), front.isInstallRocketmqExporter(), front.getGroups());
        this.module = new RocketmqModule(arg, null);
        if (!ListTool.isEmpty(targets)) {
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targets));
            param.put("module", this.module);
            param.put("groups", this.group.getGroups());
            this.paramList.add(param);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Map<String, Object> param = MapTools.simpleMap("timeout", getTimeout());
            param.put("response", "async_playbook");
            param.put("targets", getHosts(targetsMap.get(key)));
            param.put("module", this.module);
            param.put("groups", this.group.getGroups());
            if ("noworkergroup".equalsIgnoreCase(key)) {
                param.put(key, key);
            } else {
                param.put("groupId", key);
            }
            paramList.add(param);
        }
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof RocketmqInput) {
            this.input = (RocketmqInput) param;
            // 处理主机分组
            handleTargetsMap(input.getTarget().getData());
            buildParam();
        } else {
            log.error("rocketmq install task param illegel!!");
            throw new NodeFormatException("rocketmq install task param illegel!!");
        }
    }
}