package com.bocloud.tse.scheduler.worker.software;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.bocloud.tse.constant.ExecuteTypeConstant;
import com.bocloud.tse.core.domain.input.Target;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.data.redis.core.StringRedisTemplate;

import com.bocloud.tse.constant.CacheConstant;
import com.bocloud.tse.core.exception.NodeFormatException;
import com.bocloud.tse.core.module.Module;
import com.bocloud.tse.model.WorkerHost;
import com.bocloud.tse.scheduler.worker.AbstractWorker;
import com.bocloud.tse.scheduler.worker.WorkerDispatcher;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public abstract class SoftwareWorker extends AbstractWorker {
    /**
     * 目标主机数量经验值
     */
    int expNum = 100;
    WorkerDispatcher dispatcher;
    List<WorkerHost> targets;
    Map<String, List<WorkerHost>> targetsMap;
    List<Map<String, Object>> paramList;
    Module<?> module;

    public SoftwareWorker(String code, String nodeId, RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate,
        ServiceFactory serviceFactory, LockFactory lockFactory, Integer timeout) {
        super(code, nodeId, rabbitAdmin, redisTemplate, serviceFactory, lockFactory, timeout);
        this.targets = new ArrayList<>();
        this.paramList = new ArrayList<>();
        this.targetsMap = Maps.newHashMap();
        this.dispatcher = new WorkerDispatcher(rabbitAdmin, redisTemplate, lockFactory, code);
    }

    abstract void buildParam() throws NodeFormatException;

    @Override
    protected GeneralResult<?> call(String instanceId) {
        GeneralResult<?> result;
        try {
            // 任务分发并获取整体结果
            result = dispatcher.execute(paramList, getTimeout(), getNodeId(), instanceId);
        } catch (Exception e) {
            result = GeneralResult.FAILED(e.getMessage());
        }
        if (log.isDebugEnabled()) {
            log.debug("task result is {}", result);
        }
        return result;
    }

    /**
     * 处理主机网段-分组-worker关系
     *
     * @param hosts
     */
    public void handleTargetsMap(List<WorkerHost> hosts) {
        // 得到网段-主机MAP
        Map<String, List<WorkerHost>> segmengMap = new HashMap<>();
        for (WorkerHost host : hosts) {
            String key = CacheConstant.SEGMENT_ + host.getSegmentId();
            if (segmengMap.containsKey(key)) {
                segmengMap.get(key).add(host);
            } else {
                List<WorkerHost> list = new ArrayList<>();
                list.add(host);
                segmengMap.put(key, list);
            }
        }
        // 循环遍历得到分组-主机MAP
        Iterator<String> iterator = segmengMap.keySet().iterator();
        Map<String, List<WorkerHost>> groupMap = new HashMap<>();
        List<WorkerHost> nogroupList = new ArrayList<>();
        while (iterator.hasNext()) {
            String key = iterator.next();
            String value = this.getRedisTemplate().opsForValue().get(key);
            // 没有分组信息网段对应主机全部放在无分组列表中
            if (StringUtils.isEmpty(value)) {
                nogroupList.addAll(segmengMap.get(key));
                continue;
            }
            if (groupMap.containsKey(value)) {
                groupMap.get(value).addAll(segmengMap.get(key));
            } else {
                List<WorkerHost> list = new ArrayList<>();
                list.addAll(segmengMap.get(key));
                groupMap.put(value, list);
            }
        }
        // 添加无分组-主机键值对
        if (!ListTool.isEmpty(nogroupList)) {
            groupMap.put("noworkergroup", nogroupList);
        }
        Iterator<String> groupIterator = groupMap.keySet().iterator();
        while (groupIterator.hasNext()) {
            String key = groupIterator.next();
            if (targetsMap.containsKey(key)) {
                targetsMap.get(key).addAll(groupMap.get(key));
            } else {
                targetsMap.put(key, groupMap.get(key));
            }
        }
    }



    /**
     * 处理主机网段-分组-worker关系
     *
     * @param hosts
     */
    public void handleTargetsMap( String executeType, List<WorkerHost> hosts) {

        if (ExecuteTypeConstant.SALTSTACK.equalsIgnoreCase(executeType)) {
            Map<String, List<WorkerHost>> saltMap = new HashMap<>(16);
            for (WorkerHost host : hosts) {
                String key = CacheConstant.SALTMASTER_ + host.getSaltId();
                if (saltMap.containsKey(key)) {
                    saltMap.get(key).add(host);
                } else {
                    List<WorkerHost> list = new ArrayList<>();
                    list.add(host);
                    saltMap.put(key, list);
                }
            }
            Iterator<String> saltIterator = saltMap.keySet().iterator();
            while (saltIterator.hasNext()) {
                String key = saltIterator.next();
                if (targetsMap.containsKey(key)) {
                    targetsMap.get(key).addAll(saltMap.get(key));
                } else {
                    targetsMap.put(key, saltMap.get(key));
                }
            }
            return;
        }

        // 得到网段-主机MAP
        Map<String, List<WorkerHost>> segmengMap = new HashMap<>();
        for (WorkerHost host : hosts) {
            String key = CacheConstant.SEGMENT_ + host.getSegmentId();
            if (segmengMap.containsKey(key)) {
                segmengMap.get(key).add(host);
            } else {
                List<WorkerHost> list = new ArrayList<>();
                list.add(host);
                segmengMap.put(key, list);
            }
        }
        // 循环遍历得到分组-主机MAP
        Iterator<String> iterator = segmengMap.keySet().iterator();
        Map<String, List<WorkerHost>> groupMap = new HashMap<>();
        List<WorkerHost> nogroupList = new ArrayList<>();
        while (iterator.hasNext()) {
            String key = iterator.next();
            String value = this.getRedisTemplate().opsForValue().get(key);
            // 没有分组信息网段对应主机全部放在无分组列表中
            if (StringUtils.isEmpty(value)) {
                nogroupList.addAll(segmengMap.get(key));
                continue;
            }
            if (groupMap.containsKey(value)) {
                groupMap.get(value).addAll(segmengMap.get(key));
            } else {
                List<WorkerHost> list = new ArrayList<>();
                list.addAll(segmengMap.get(key));
                groupMap.put(value, list);
            }
        }
        // 添加无分组-主机键值对
        if (!ListTool.isEmpty(nogroupList)) {
            groupMap.put("noworkergroup", nogroupList);
        }
        Iterator<String> groupIterator = groupMap.keySet().iterator();
        while (groupIterator.hasNext()) {
            String key = groupIterator.next();
            if (targetsMap.containsKey(key)) {
                targetsMap.get(key).addAll(groupMap.get(key));
            } else {
                targetsMap.put(key, groupMap.get(key));
            }
        }
    }


    public List<WorkerHost> getHosts(List<WorkerHost> hosts) {
        List<WorkerHost> list = new ArrayList<>();
        if (!ListTool.isEmpty(hosts)) {
            List<String> ips = new ArrayList<>();
            for (WorkerHost host : hosts) {
                if (!ips.contains(host.getHost())) {
                    ips.add(host.getHost());
                    list.add(host);
                }
            }
        }
        return list;
    }

    public void processBecome(Target target){
        if (null == target) {
            return;
        }
        if (!ListTool.isEmpty(targets)) {
            processBecome(target,this.targets);
            return;
        }
        Iterator<String> iterator = targetsMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            List<WorkerHost> hosts = targetsMap.get(key);
            processBecome(target,hosts);
        }

    }

    private void processBecome(Target target, List<WorkerHost> hosts){
        Boolean become = null != target.getBecome() ? target.getBecome() : target.getSudo();
        for (WorkerHost host : hosts) {
            Boolean hostBecome = null != host.getBecome() ? host.getBecome() : (null != host.getSudo() ? host.getSudo() : become);
            if (null == hostBecome || !hostBecome) {
                host.setBecome(false);
                host.setBecomeMethod(null);
                host.setBecomeUser(null);
                host.setBecomePass(null);
                continue;
            }
            host.setBecome(true);
            host.setBecomeUser(null != host.getBecomeUser() ? host.getBecomeUser() : target.getBecomeUser());
            host.setBecomePass(null != host.getBecomePass() ? host.getBecomePass() : target.getBecomePass());
            host.setBecomeMethod(host.getBecomeMethod());
        }
    }
}