package com.bocloud.tse.scheduler.worker.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.model.HostResult;
import com.bocloud.tse.model.WorkerHost;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

@Slf4j
public class ResultMessageHandler {

    public static GeneralResult<?> handleResult(GeneralResult<?> invoke, String instance, List<WorkerHost> targets) {
        if (null == invoke.getData()) {
            String message = "返回结果异常：未返回任何机器任务信息；worker可能处于异常状态，未进行任务执行";
            return handleError(message, instance, targets);
        }
        List<HostResult> results = buildHostResult(invoke);
        if (ListTool.isEmpty(results)) {
            String message = "返回结果异常：未返回任何机器任务信息；worker可能处于异常状态，未进行任务执行";
            for (WorkerHost target : targets) {
                results.add(new HostResult(false, message, target.getHost(), target.getRawName(), target.getPort(),
                        target.getUser(), instance));
            }
            return new GeneralResult<>(false, results, invoke.getMessage());
        }
        for (HostResult result : results) {
            result.setWorkerInstance(instance);
        }
        return new GeneralResult<>(invoke.isSuccess(), results, invoke.getMessage());
    }

    public static GeneralResult<List<HostResult>> handleError(String message, String instance, List<WorkerHost> targets) {
        List<HostResult> results = new ArrayList<>();
        for (WorkerHost target : targets) {
            results.add(new HostResult(false, message, target.getHost(), target.getRawName(), target.getPort(),
                    target.getUser(), instance));
        }
        return new GeneralResult<>(false, results, message);
    }

    public static List<HostResult> buildHostResult(GeneralResult<?> result) {
        List<HostResult> results = new ArrayList<>();
        if (null != result.getData()) {
            try {
                results = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
            } catch (Exception e) {
                log.error("build host result error!", e);
            }
        }
        return results;
    }

}
