package com.bocloud.tse.scheduler.worker.utils;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import org.apache.commons.lang3.StringUtils;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class SaltClientCache {

    private Map<String, SaltClientModel> cache;
    static ReentrantReadWriteLock rwLock = new ReentrantReadWriteLock();
    static Lock readLock = rwLock.readLock();
    static Lock writeLock = rwLock.writeLock();

    private SaltClientCache() {
        if (null != FactoryContainer.instance) {// 防止利用反射机制实现多个对象的漏洞
            throw new RuntimeException("SaltClientCache遭受反射攻击！");
        }
        cache = new ConcurrentHashMap<>();
    }

    public void update(String key, SaltClientModel value) {
        processNull();
        if (StringUtils.isEmpty(key)) {
            return;
        }
        writeLock.lock();
        try {
            cache.put(key, value);
        } catch (Exception e) {
            log.error("update key【{}】 salt client error ", key, e);
        } finally {
            writeLock.unlock();
        }
    }

    public SaltClientModel getClient(String key) {
        readLock.lock();
        try {
            processNull();
            return cache.get(key);
        } catch (Exception e) {
            log.error("get salt client error:", e);
            return null;
        } finally {
            readLock.unlock();
        }
    }

    private void processNull() {
        if (null == cache) {
            cache = new ConcurrentHashMap<>(16, 0.75F);
        }
    }

    public static SaltClientCache getInstance() {
        return FactoryContainer.instance;
    }

    private static class FactoryContainer {
        private static SaltClientCache instance = new SaltClientCache();
    }
}
