package com.bocloud.tse.repository;

import com.bocloud.tse.entity.NodeRelation;
import com.bocloud.tse.enums.Status;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 任务主机DAO接口实现
 */
@Repository("nodeRelationRepository")
public class NodeRelationRepository extends BasicGenericDao<NodeRelation, Long> {

    public NodeRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean batchSave(List<NodeRelation> relations) {
        return this.batchSave(relations, relations.size());
    }

    public boolean updateStatus(Long instanceId, Long source, String status) {
        String sql =
            "update task_node_relation set status = :status where source = :source and instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("source", source);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public List<Long> getReadyNodes(Long instanceId) {
        String sql =
            "select distinct a.source from task_node_relation a where a.instance_id = :instanceId and a.status = '"
                + Status.Node.READY.name()
                + "' and a.source not in (select distinct b.target from task_node_relation b where b.instance_id = a.instance_id and b.status not in ('"
                + Status.Node.SUCCESS.name() + "','" + Status.Node.FAILED.name() + "','" + Status.Node.CANCELED.name()
                + "','" + Status.Node.EXCEPTION.name() + "','" + Status.Node.NOREADY.name() + "','"
                + Status.Node.SKIPPED.name() + "'))";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        List<NodeRelation> list = this.list(NodeRelation.class, sql, params);
        List<Long> nodes = new ArrayList<>();
        for (NodeRelation relation : list) {
            nodes.add(relation.getSource());
        }
        return nodes;
    }

    public List<Long> getUndoneNodes(Long instanceId) {
        String sql = "select distinct a.source from task_node_relation a where a.status = '" + Status.Node.READY.name()
            + "' and a.instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        List<NodeRelation> list = this.list(NodeRelation.class, sql, params);
        List<Long> nodes = new ArrayList<>();
        for (NodeRelation relation : list) {
            nodes.add(relation.getSource());
        }
        return nodes;
    }

    public List<Long> getEndNodes(Long instanceId) {
        String sql = "select distinct a.target from task_node_relation a where a.instance_id = :instanceId"
            + " and a.target not in (select distinct b.source from task_node_relation b where b.instance_id = a.instance_id)";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        List<NodeRelation> list = this.list(NodeRelation.class, sql, params);
        List<Long> nodes = new ArrayList<>();
        for (NodeRelation relation : list) {
            nodes.add(relation.getTarget());
        }
        return nodes;
    }

}
