package com.bocloud.tse.repository;

import com.bocloud.tse.entity.TaskFailRecord;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 任务失败记录
 * 
 * @author hongqingyu
 */
@Repository("taskFailRecordRepository")
public class TaskFailRecordRepository extends BasicGenericDao<TaskFailRecord, Long> {
    public TaskFailRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 查询指定记录
     * 
     * @param instanceId
     *            任务实例的id
     * @return
     */
    public List<TaskFailRecord> list(Long instanceId) {
        String sql = "select a.* from task_fail_record a where instance_id =:instanceId ";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        return this.list(TaskFailRecord.class, sql, params);
    }

    /**
     * 根据实例id删除错误记录
     * 
     * @param id
     */
    public void delete(Long id) {
        String sql = "delete from task_fail_record where id =:id ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        this.execute(sql, params);
    }

    /**
     * 根据实例id删除错误记录
     * 
     * @param instanceId
     */
    public void deleteByInstanceId(Long instanceId) {
        String sql = "delete from task_fail_record where instance_id =:instanceId ";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        this.execute(sql, params);
    }

}
