package com.bocloud.tse.repository;

import com.bocloud.tse.entity.TaskInstance;
import com.bocloud.tse.enums.Status;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 任务实例接口实现
 */
@Repository("taskInstanceRepository")
public class TaskInstanceRepository extends JdbcGenericDao<TaskInstance, Long> {

    public TaskInstanceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public TaskInstance query(Long id) {
        String sql = "select a.* from task_instance a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<TaskInstance> list = this.list(TaskInstance.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean remove(Long id, Long userId) {
        String sql =
            "update task_instance set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update task_instance set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", status);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskInstance> listByHost(Status.Inst status, String executeHost, String excludeHost) {
        String sql = "select * from task_instance where status = :status ";
        Map<String, Object> params = MapTools.simpleMap("status", status.name());
        if (StringUtils.hasText(executeHost)) {
            sql += " and execute_host = :executeHost ";
            params.put("executeHost", executeHost);
        }
        if (StringUtils.hasText(excludeHost)) {
            sql += " and execute_host != :excludeHost";
            params.put("excludeHost", excludeHost);
        }
        return list(TaskInstance.class, sql, params);
    }

    public List<TaskInstance> listExpInstance(String executeHost, String excludeHost) {
        StringBuffer sqlBuffer = new StringBuffer(
            "SELECT * FROM task_instance WHERE is_deleted=0 AND status IN ('RUNNING','SUSPENDED','CANCELING','WAITTING') ");
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(executeHost)) {
            sqlBuffer.append("AND execute_host = :executeHost ");
            params.put("executeHost", executeHost);
        }
        if (StringUtils.hasText(excludeHost)) {
            sqlBuffer.append("AND execute_host != :excludeHost ");
            params.put("excludeHost", excludeHost);
        }
        // sqlBuffer.append("OR execute_host IS NULL");
        return list(TaskInstance.class, sqlBuffer.toString(), params);
    }

    public List<TaskInstance> listByTask(Long taskId) {
        String sql = "select a.* from task_instance a where a.is_deleted = 0 and a.task_id = :taskId";
        Map<String, Object> params = MapTools.simpleMap("taskId", taskId);
        return this.list(TaskInstance.class, sql, params);
    }

    public boolean updateTime(Long id) throws Exception {
        String sql = "update task_instance set gmt_create = :gmtCreate, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtCreate", new Date());
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

}
