package com.bocloud.tse.repository;

import com.bocloud.tse.entity.TaskNode;
import com.bocloud.tse.enums.Status;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务节点DAO接口实现类
 */
@Repository("taskNodeRepository")
public class TaskNodeRepository extends JdbcGenericDao<TaskNode, Long> {

    public TaskNodeRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskNode> listByInstance(Long instanceId) {
        String sql =
            "select * from task_node a where a.is_deleted = 0 and a.instance_id = :instanceId order by a.gmt_start asc";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        return this.list(TaskNode.class, sql, params);
    }

    public TaskNode query(Long id) {
        String sql = "select * from task_node a where a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<TaskNode> list = this.list(TaskNode.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update task_node set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public boolean hasReady(Long instanceId) {
        String sql =
            "select a.* from task_node a where a.is_deleted = 0 and a.status = 'READY' and a.instance_id = :instanceId";
        Map<String, Object> paramMap = MapTools.simpleMap("instanceId", instanceId);
        List<TaskNode> nodes = this.list(TaskNode.class, sql, paramMap);
        return !nodes.isEmpty();
    }

    public List<Long> getFreeNodes(Long instanceId) {
        String sql = "select a.id from task_node a where a.status = '" + Status.Node.READY.name()
            + "' and a.instance_id = :instanceId and a.id not in "
            + "(select DISTINCT b.source from task_node_relation b where b.instance_id = a.instance_id "
            + "union select DISTINCT c.target from task_node_relation c where c.instance_id = a.instance_id )";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        List<TaskNode> list = this.list(TaskNode.class, sql, params);
        List<Long> nodes = new ArrayList<>();
        for (TaskNode node : list) {
            nodes.add(node.getId());
        }
        return nodes;
    }

    public List<Long> getLastNodes(Long instanceId) {
        String sql = "select node.* from task_node node where node.instance_id = :instanceId and node.status= 'READY'"
            + " and node.id not in (select relation.source from task_node_relation relation where relation.instance_id = node.instance_id)"
            + " and node.id in (select r.target from task_node_relation r where r.instance_id = node.instance_id and r.status in ('SUCCESS','NOREADY','SKIPPED')"
            + " and r.target not in (select re.target from task_node_relation re where re.instance_id = node.instance_id and re.status != 'SUCCESS' and re.status != 'NOREADY' and re.status != 'SKIPPED'))";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        List<TaskNode> list = this.list(TaskNode.class, sql, params);
        List<Long> nodes = new ArrayList<>();
        for (TaskNode node : list) {
            nodes.add(node.getId());
        }
        return nodes;
    }

}
