package com.bocloud.tse.repository;

import com.bocloud.tse.entity.Task;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务DAO接口实现类
 */
@Repository("taskRepository")
public class TaskRepository extends JdbcGenericDao<Task, Long> {

    public TaskRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public Task query(Long id) {
        String sql = "select a.* from task a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<Task> list = this.list(Task.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql = "update task set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public Task get(String uuid) {
        String sql = "select a.* from task a where a.is_deleted = 0 and a.uuid = :uuid";
        Map<String, Object> paramMap = MapTools.simpleMap("uuid", uuid);
        List<Task> list = this.list(Task.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateExecute(Long id, Date date) {
        String sql = "update task set gmt_execute = :gmtExecute where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtExecute", date);
        return this.execute(sql, params) > 0;
    }

    public List<Task> listSchedule() {
        String sql =
            "select a.* from task a where a.is_deleted = 0 and a.code = 'scheduled' and (a.status = 'RUNNING' or a.status = 'WAITTING')";
        return this.list(Task.class, sql);
    }

}
