package com.bocloud.tse.service;

import java.util.List;
import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.tse.entity.Command;
import com.bocloud.tse.intf.CommandService;
import com.bocloud.tse.repository.CommandRepository;
import com.google.common.collect.Lists;
import com.megatron.common.utils.JSONTools;

import lombok.extern.slf4j.Slf4j;

/**
 * 命令service接口实现
 */
@Slf4j
@Service("commandService")
public class CommandServiceImpl implements CommandService {

    @Autowired
    private CommandRepository commandRepository;
    private static Pattern pattern = Pattern.compile("([\\w]+)");

    /**
     * 获取脚本中高危命令的ID
     */
    @Override
    public List<Long> getCommands(String content) {
        List<Long> list = Lists.newArrayList();
        JSONObject jsonObj = JSONTools.isJSONObj(content);
        JSONObject fronts = jsonObj.getJSONObject("fronts");
        // 提取内容
        if (fronts != null) {
            String contentStr = fronts.getString("content");
            content = StringUtils.hasText(contentStr) ? contentStr : content;
        }
        try {
            // 按换行 分割
            String[] contents = content.split("\\n");
            // 从数据库里面查询出高危命令
            List<Command> commands = commandRepository.list();
            if (commands.isEmpty()) {
                return list;
            }

            for (Command command : commands) {
                for (String str : contents) {
                    // 处理单行数据
                    String comContent = command.getContent().trim();
                    MatchResult matchResult = pattern.matcher(str).toMatchResult();
                    List<String> commandAndOptionList = getCommandPerLine(matchResult);
                    int index = str.indexOf("#");
                    if (index == -1) {
                        // 没有注释的时候
                        if (commandAndOptionList.contains(comContent)) {
                            list.add(command.getId());
                        }
                    } else {
                        // 有注释的时候
                        if (commandAndOptionList.contains(comContent) && str.indexOf("#") > str.indexOf(comContent)) {
                            list.add(command.getId());
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("get deadly commandID fail:", e);
        }
        return list;
    }

    private List<String> getCommandPerLine(MatchResult matchResult) {
        List<String> list = Lists.newArrayList();
        while (((Matcher)matchResult).find()) {
            list.add(matchResult.group());
        }
        return list;
    }

}
