package com.bocloud.tse.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.tse.entity.TaskNode;
import com.bocloud.tse.intf.TaskNodeService;
import com.bocloud.tse.repository.NodeRelationRepository;
import com.bocloud.tse.repository.TaskNodeRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;

import lombok.extern.slf4j.Slf4j;

/**
 * 记录抽象Service接口实现类
 */
@Slf4j
@Service("taskNodeService")
public class TaskNodeServiceImpl implements TaskNodeService {

    @Autowired
    private TaskNodeRepository taskNodeRepository;
    @Autowired
    private NodeRelationRepository nodeRelationRepository;

    @Override
    public GeneralResult<Void> modify(TaskNode node) {
        try {
            Boolean result = taskNodeRepository.update(node);
            return result ? new GeneralResult<>(true, "修改任务节点成功") : new GeneralResult<>(false, "修改任务节点失败");
        } catch (Exception e) {
            log.error("Modify job record fail:", e);
            return new GeneralResult<>(false, "修改任务节点失败");
        }
    }

    @Override
    public TaskNode get(Long id) {
        try {
            return taskNodeRepository.query(id);
        } catch (Exception e) {
            log.error("Get job record fail:", e);
            return null;
        }
    }

    @Override
    public List<TaskNode> getReadyNodes(Long instanceId) {
        List<TaskNode> nodes = Lists.newArrayList();
        try {
            List<Long> nodeIds = this.nodeRelationRepository.getReadyNodes(instanceId);
            for (Long nodeId : nodeIds) {
                TaskNode node = this.taskNodeRepository.query(nodeId);
                nodes.add(node);
            }
            return nodes;
        } catch (Exception e) {
            log.error("get ready node list fail:", e);
            return null;
        }
    }

    @Override
    public List<TaskNode> getFreeNodes(Long instanceId) {
        List<TaskNode> nodes = Lists.newArrayList();
        try {
            List<Long> nodeIds = this.taskNodeRepository.getFreeNodes(instanceId);
            for (Long nodeId : nodeIds) {
                TaskNode node = this.taskNodeRepository.query(nodeId);
                nodes.add(node);
            }
        } catch (Exception e) {
            log.error("get ready free node list fail:", e);
        }
        return nodes;
    }

    @Override
    public Result updateRelation(Long instanceId, Long source, String status) {
        try {
            boolean result = this.nodeRelationRepository.updateStatus(instanceId, source, status);
            return result ? new GeneralResult<>(true, "节点状态更新成功") : new GeneralResult<>(false, "节点状态更新失败");
        } catch (Exception e) {
            log.error("update node relation status fail:", e);
            return new GeneralResult<>(false, "关系状态更新失败");
        }
    }

    @Override
    public List<TaskNode> getUndoneNodes(Long instanceId) {
        try {
            List<Long> nodeIds = this.nodeRelationRepository.getUndoneNodes(instanceId);
            List<Long> endNodeIds = this.nodeRelationRepository.getEndNodes(instanceId);
            if (!ListTool.isEmpty(endNodeIds)) {
                nodeIds.addAll(endNodeIds);
            }
            List<TaskNode> nodes = Lists.newArrayList();
            for (Long nodeId : nodeIds) {
                TaskNode node = this.taskNodeRepository.query(nodeId);
                if ("READY".equals(node.getStatus())) {
                    nodes.add(node);
                }
            }
            return nodes;
        } catch (Exception e) {
            log.error("get ready node list fail:", e);
            return null;
        }
    }

    @Override
    public List<TaskNode> getNodes(Long instanceId) {
        try {
            return this.taskNodeRepository.listByInstance(instanceId);
        } catch (Exception e) {
            log.error("Get nodes list fail:", e);
            return Lists.newArrayList();
        }
    }

    @Override
    public List<TaskNode> getLastNodes(Long instanceId) {
        List<TaskNode> nodes = Lists.newArrayList();
        try {
            List<Long> nodeIds = this.taskNodeRepository.getLastNodes(instanceId);
            for (Long nodeId : nodeIds) {
                TaskNode node = this.taskNodeRepository.query(nodeId);
                nodes.add(node);
            }
            return nodes;
        } catch (Exception e) {
            log.error("get last node fail:", e);
            return null;
        }
    }

    @Override
    public Boolean hasReady(Long instanceId) {
        try {
            return this.taskNodeRepository.hasReady(instanceId);
        } catch (Exception e) {
            log.error("query ready nodes fail:", e);
            return false;
        }
    }

    @Override
    public Result updateStatus(Long id, String status, Long userId) {
        try {
            boolean result = this.taskNodeRepository.updateStatus(id, status, userId);
            return result ? new GeneralResult<>(true, "节点状态更新成功") : new GeneralResult<>(false, "节点状态更新失败");
        } catch (Exception e) {
            log.error("update node relation status fail:", e);
            return new GeneralResult<>(false, "关系状态更新失败");
        }
    }

}
