package com.bocloud.tse.service.internal;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;

/**
 * @author lyy
 * @date 2023年04月03日 16:05
 */
@Component
public class TseInternalService implements InitializingBean {

    @Autowired
    private CurrentService currentService;

    private static final Service SERVICE = Service.create(BoCloudService.TSE.getName());
    private final Map<String, Object> headers = new HashMap<>(4);

    public GeneralResult<JSONObject> getStatus(String instance) {
        String url = "http://" + instance + "/v1/status/task/total";
        RemoteService service = new RemoteService(SERVICE, new ServiceParam(null, headers), true, url, HttpMethod.GET);
        GeneralResult<?> result = service.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data), "success");
    }

    public GeneralResult<Boolean> getEngineStatus(String instance) {
        String url = "http://" + instance + "/v1/engines/status";
        RemoteService service = new RemoteService(SERVICE, new ServiceParam(null, headers), true, url, HttpMethod.GET);
        GeneralResult<?> result = service.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = null == result.getData() ? null : String.valueOf(result.getData());
        return new GeneralResult<Boolean>(true, Boolean.valueOf(data), "success");
    }

    public GeneralResult<?> listenerAction(String instance, String action) {
        String url = "http://" + instance + "/v1/engines/action";
        Map<String, Object> params = MapTools.simpleMap("instance", instance);
        params.put("action", action);
        RemoteService service =
            new RemoteService(SERVICE, new ServiceParam(params, headers), true, url, HttpMethod.POST);
        service.setDataFormat(PostDataFormat.RAW);
        return service.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
